<script setup>

import JetModal from '@/Jetstream/Modal.vue';
import Datepicker from '@vuepic/vue-datepicker';
import '@vuepic/vue-datepicker/dist/main.css';
import JetDropdown from '@/Jetstream/Dropdown.vue';
import { startOfDay, endOfDay, subDays, endOfMonth, endOfYear, startOfMonth, startOfYear, subMonths } from 'date-fns';
import DK_Input from '@/components/dk_Input.vue';
import DK_Button from '@/components/dk_Button.vue';
import { Head, Link, useForm } from '@inertiajs/inertia-vue3';
import Highcharts from 'highcharts';
import VueMultiselect from 'vue-multiselect';
import VueTimepicker from 'vue3-timepicker';
// import slider from "vue3-slider";
// import 'vue3-timepicker/dist/VueTimepicker.css';

let AppLayout;

if (window.location.origin.indexOf('localhost:8000') !== -1 || window.location.origin.indexOf('water.baltoro.app') !== -1) {
    AppLayout = require('@/Layouts/AppLayoutWater.vue').default;
} else {
    AppLayout = require('@/Layouts/AppLayoutCity.vue').default;
}
</script>

<style>

/* START DP */
.dp__theme_light {
    --dp-background-color: rgba(0, 57, 93, 0.95);
    --dp-text-color: #e3e3e3;
    --dp-hover-color: rgba(0, 255, 255, 0.2);
    --dp-hover-text-color: #FFFAFA;
    --dp-hover-icon-color: #e3e3e3;
    --dp-primary-color: rgba(0, 255, 255, 0.5);
    --dp-primary-text-color: #FFFAFA;
    --dp-secondary-color: #959595;
    --dp-border-color: #FFFAFA;
    --dp-menu-border-color: #FFFAFA;
    --dp-border-color-hover: #FFFAFA;
    --dp-disabled-color: #959595;
    --dp-scroll-bar-background: #f3f3f3;
    --dp-scroll-bar-color: #959595;
    --dp-success-color: rgba(5, 255, 243, 0.7);
    --dp-success-color-disabled: rgba(5, 255, 243, 0.7);
    --dp-icon-color: #e3e3e3;
    --dp-danger-color: #e6002e;
    --dp-highlight-color: rgba(25, 118, 210, 0.1);
}

.dp__input {
    background-color: transparent !important;
    border: none !important;
}

/* END DP */

/* START SWITCH STYLING */
.switch {
    position: relative;
    display: inline-block;
    width: 28px;
    height: 12px;
}

.switch input {
    opacity: 0;
    width: 0;
    height: 0;
}

.slider {
    position: absolute;
    cursor: pointer;
    top: 0;
    left: 0;
    right: 0;
    bottom: 0;
    background-color: rgba(200, 200, 200, 1);
    -webkit-transition: .4s;
    transition: .4s;
}

.slider-2 {
    position: absolute;
    cursor: pointer;
    top: 0;
    left: 0;
    right: 0;
    bottom: 0;
    background-color: rgba(180, 120, 0, 1);
    -webkit-transition: .4s;
    transition: .4s;
}

.slider:before,
.slider-2:before {
    position: absolute;
    content: "";
    height: 16px;
    width: 16px;
    left: -2px;
    bottom: -2px;
    background-color: white;
    -webkit-transition: .4s;
    transition: .4s;
}

input:checked+.slider,
input:checked+.slider-2 {
    background-color: #0098db;
}

input:focus+.slider,
input:focus+.slider-2 {
    box-shadow: 0 0 1px #0098db;
}

input:checked+.slider:before,
input:checked+.slider-2:before {
    -webkit-transform: translateX(16px);
    -ms-transform: translateX(16px);
    transform: translateX(16px);
}

.slider.round,
.slider-2.round {
    border-radius: 16px;
}

.slider.round:before,
.slider-2.round:before {
    border-radius: 50%;
}

/* END SWITCH STYLING */

/* START CHECKBOXES STYLING */
input[type="checkbox"] {
    background-color: transparent;
    cursor: pointer;
    display: inline-block;
    height: 12px;
    position: relative;
    width: 12px;
    -webkit-appearance: none;
}

input[type="checkbox"]:after {
    background-color: rgba(0, 42, 80, 0.25);
    border-width: 1px;
    border-color: rgba(175, 175, 175, 0.5);
    border-radius: 3px;
    content: '';
    display: block;
    height: 12px;
    position: relative;
    width: 12px;
}

input[type="checkbox"]:checked:after {
    background-color: rgba(255, 250, 250, 0.7);
}

/* END CHECKBOXES STYLING */

/* ANIMATION-Ping STYLING */

@keyframes ping {
  0%, 10% {
    transform: rotate(-8deg);
  }
  10%, 20% {
    transform: rotate(8deg);
  }
  20%, 30% {
    transform: rotate(-8deg);
  }
  30%, 40% {
    transform: rotate(8deg);
  }
  40%, 50% {
    transform: rotate(-8deg);
  }
  50%, 60% {
    transform: rotate(8deg);
  }
  60%, 70% {
    transform: rotate(-8deg);
  }
  70%, 80% {
    transform: rotate(8deg);
  }
  80%, 90% {
  }
  90%, 100% {
  }
}

/* TIMEPICKER */

.vue__time-picker input.vue__time-picker-input {
    border: 1px solid rgba(175, 175, 175, 0.5);
    color: rgb(225, 225, 225);
    background: transparent;
    width: 10em;
    height: 28px;
    font-size: 1em;
    border-radius: 0.375rem;
    text-align: center;
    cursor: pointer;
}

.vue__time-picker .dropdown {
    position: absolute;
    border-radius: 0.375rem;
    z-index: 5;
    left: 0;
    background: #001e3c;
    width: 10em;
    height: 10em;
    border: 1px solid rgba(175, 175, 175, 0.5);
    font-weight: 400;
}

.vue__time-picker .dropdown ul li:not([disabled]).active,
.vue__time-picker .dropdown ul li:not([disabled]).active:focus,
.vue__time-picker .dropdown ul li:not([disabled]).active:hover {
    background: #0080E8;
    color: #fff;
    border-radius: 0.375rem;
}

.vue__time-picker .dropdown ul li {
    list-style: none;
    text-align: center;
    padding: 0.3em 0;
    color: rgba(175, 175, 175, 0.5);
}

.vue__time-picker .dropdown ul li:not(.hint):not([disabled]):focus,
.vue__time-picker .dropdown ul li:not(.hint):not([disabled]):hover {
    color: rgb(225, 225, 225);
    cursor: pointer;
}

.vue__time-picker .dropdown ul.apms,
.vue__time-picker .dropdown ul.minutes,
.vue__time-picker .dropdown ul.seconds {
    border-left: 0px solid #fff;
}

.vue__time-picker .dropdown ul {
    margin-right: 0.15rem;
    padding: 0;
    list-style: none;
    outline: 0;
    outline-color: initial;
    outline-style: initial;
    outline-width: 0px;
    flex: 1 1 0.00001px;
    overflow-x: hidden;
    overflow-y: auto;
}

/* END WIZARD BAR */

.fa-trash-can:hover {
    color: #e6002e !important;
}

.fa-pen:hover {
    color: #FFFAFA !important;
}

.fa-filter:hover {
    color: #FFFAFA !important;
}

.fa-pen:hover {
    color: #FFFAFA !important;
}

.fa-clock:hover {
    color: #FFFAFA !important;
}

.fa-plus:hover {
    color: #FFFAFA !important;
}

/* Estilos para el input de tipo "number" */
input[type="number"] {
  width: 100%;
  border-bottom: 1px solid rgba(175, 175, 175, 0.5);
  transition: border-color 0.3s ease;
  outline: none;
  background-color: transparent;
  width: 1.75rem;
  text-align: center;
  -webkit-appearance: none;
  margin: 0;
}

input[type="number"]:focus {
  border-color: rgba(225, 225, 225, 0.7);
}

input[type="number"]::-webkit-inner-spin-button,
input[type="number"]::-webkit-outer-spin-button {
  -webkit-appearance: none;
  margin: 0;
}

</style>
<template>
    <AppLayout title="Alerts">
        <loader-component></loader-component>
        <div v-if="this.screen_width >= 768" style="height: calc(100vh - 4.75rem);" class="bg-darkblue-background pl-6 z-0 cursor-default flex justify-start gap-6 overflow-y-hidden select-none">

            <!-- START ALERTS LOG -->
            <div class="w-2/3 2xl:w-3/4 py-4 flex flex-col justify-start h-full">

                <div class="flex justify-between items-center pb-4">
                    <p class="text-white-snow font-light text-center text-2xl">{{ this.$t("general." + "Alerts log") }}</p>

                    <div class="bg-blue-searchbar w-fit rounded-md shadow-lg border border-gray-semimedium flex items-center">

                        <input type="text" v-on:keyup="set_SearchFilter()"
                            :placeholder='this.$t("text." + "Search in logs")' id="id_searchbar"
                            class="focus-visible:outline-0 text-md text-left px-4 py-2 text-white-snow font-semilight rounded-l-md bg-transparent w-[20vw]">

                        <!--<div class="border-l border-gray-semimedium px-5 flex justify-center items-center w-12 h-9">
                                        <i class="fa-solid fa-filter" style="font-size:1rem; color:rgba(255, 250, 250, 1);"></i>
                                    </div>-->

                        <div class="border-l border-gray-semimedium pl-2 flex justify-arround items-center">
                            <Datepicker v-bind:class="{ 'w-12': !DP_open, 'w-[21rem]': DP_open }" id="interval_date"
                                v-model="date" :presetRanges="this.presetRanges" vertical range
                                @update:modelValue="set_FilterDates" :clearable="false" :format="this.Format"></Datepicker>
                            <i v-if="DP_open" v-on:click="this.delete_FilterDates()" class="fa-solid fa-xmark mr-4"
                                style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                        </div>

                    </div>
                </div>

                <div style="height: calc(100vh - 10rem);"
                    class="flex flex-col justify-start gap-y-[1px] bg-darkblue-opa-container rounded-md border border-gray-semimedium">

                    <div class="py-1 flex justify-start items-center text-gray-light font-light text-center gap-x-3 border-transparent border-l-2 h-8">

                        <div class="flex flex-row items-center justify-start gap-x-6">
                            <div
                                class="flex gap-x-1 rounded-md active:bg-white-hover focus:bg-white-hover hover:bg-white-hover p-2 cursor-pointer">
                                <input type="checkbox" id="checkbox_all"
                                    v-on:click="this.set_CheckboxClick('checkbox_all', 'All_box')">
                                <JetDropdown align="left">
                                    <template #trigger>
                                        <div class="flex flex-row items-center justify-start px-0.5">
                                            <i class="fa-solid fa-caret-down"
                                                style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                                        </div>
                                    </template>
                                    <template #content>
                                        <option v-for="(selector, index) in this.group_selector" :key="index"
                                            :value="selector" class="hover:bg-cyan-opa-2"
                                            v-on:click="this.set_CheckboxClick('checkbox_all', selector)">
                                            {{ selector }}
                                        </option>
                                    </template>
                                </JetDropdown>
                            </div>
                            <i v-if="this.Global_Checkbox_State" v-on:click="this.back_Checkboxes()"
                                class="fa-solid fa-xmark cursor-pointer"
                                style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                            <i v-if="this.Global_Checkbox_State || this.Checkboxes_State"
                                v-on:click="this.set_ReadSelectedAlertLogs(false)"
                                class="fa-solid fa-eye-slash cursor-pointer"
                                style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                            <i v-if="this.Global_Checkbox_State || this.Checkboxes_State"
                                v-on:click="this.set_ReadSelectedAlertLogs(true)" class="fa-solid fa-eye cursor-pointer"
                                style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                            <i v-if="this.Global_Checkbox_State || this.Checkboxes_State"
                                v-on:click="this.open_Delete(false, 'group_alert_log')"
                                class="fa-solid fa-trash-can cursor-pointer"
                                style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                        </div>

                    </div>

                    <div class="overflow-y-scroll gap-y-[1px] flex flex-col">
                        <div v-for="(log, index) in this.alerts_log" :key="index"
                            class="hover:border-white border-transparent border-l-2 pl-2 pr-8 py-1 hover:text-white-snow hover:shadow-lg text-center flex justify-between items-center h-8 cursor-pointer font-normal"
                            v-bind:class="{ 'bg-blue-unread hover:bg-blue-unread-hover text-gray-light': log.read == 0, 'bg-blue-read hover:bg-blue-read-hover text-white-snow': log.read == 1, 'hidden': temporal_display_logs[index] != true }">

                            <div class="flex justify-start items-center w-fit">
                                <input type="checkbox" :id="'checkbox_' + log.id"
                                    v-on:click="this.set_CheckboxClick(index)">
                            </div>

                            <div v-on:click="this.set_ReadAlertLog(index)" class="flex items-center justify-evenly w-full px-6 gap-8">

                                <p class="text-left w-2/5">{{ log.description }}</p>

                                <div class="flex flex-row justify-start whitespace-nowrap text-ellipsis overflow-hidden w-2/5">
                                    <div v-for="(gateway, index_g) in this.gateways" :key="index_g">
                                        <p v-if="gateway.id == log.iddevice" class="text-left">
                                            {{ gateway.name }}
                                        </p>
                                    </div>
                                </div>

                                <p class="text-left w-1/5 whitespace-nowrap">{{ this.formatDate(log.date, 0, false) }}</p>

                            </div>

                            <div class="flex justify-end items-center w-fit"
                                v-on:click="this.open_Delete(false, 'alert_log', index)">
                                <i class="fa-solid fa-trash-can"
                                    style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                            </div>

                        </div>
                    </div>
                </div>
            </div>
            <!-- END ALERTS LOG -->

            <!-- START ALERTS SIDEBAR -->
            <div id="id_Alert_Sidebar" class="py-6 w-1/3 2xl:w-1/4 flex flex-col justify-start overflow-y-scroll overflow-x-hidden h-full border-l border-r-0 border-gray-semimedium shadow-lg bg-darkblue-opa-container duration-200 fload-right">

                <!-- START OPEN SIDEBAR ALERTS -->
                <div class="px-1">
                    <div class="flex flex-row items-center justify-between pl-1 mb-3 py-[5px]">
                        <p class="text-white-snow font-light text-center text-2xl">{{ this.$t("general." + "Alerts") }}</p>

                        <div class="flex flex-row items-center justify-around hover:bg-blue-alerts-hover rounded-md border border-gray-semimedium gap-1 px-2 py-1" v-on:click="this.open_create_Modal()">
                            <i class="fa-solid fa-plus" style="font-size:0.875rem; color:#FFFAFA;"></i>
                            <p class="text-white-snow text-sm">Create New</p>
                        </div>

                    </div>

                    <div class="flex flex-col justify-center mb-0.5 ml-1">
                        <div class="w-full flex flex-row justify-between gap-6 pr-2 pb-1">
                            <div class="w-full">
                                <p class="text-[0.6rem] font-light text-white-snow">{{ this.$t("general." + "Description") + ' ' + this.$t("general." + "and") +  ' ' + this.$t("general." + "Type" ) }}</p>
                            </div>
                            <div class="grid grid-cols-6 place-items-center gap-4 w-full">
                                <p class="text-[0.6rem] font-light text-white-snow">{{ this.$t("general." + "Filter" ) }}</p>
                                <p class="text-[0.6rem] font-light text-white-snow">{{ this.$t("general." + "Edit" ) }}</p>
                                <p class="text-[0.6rem] font-light text-white-snow">{{ this.$t("general." + "Timetable" ) }}</p>
                                <p class="text-[0.6rem] font-light text-white-snow">{{ this.$t("general." + "Active" ) }}</p>
                                <p class="text-[0.6rem] font-light text-white-snow">{{ this.$t("general." + "Delete" ) }}</p>
                                <p class="text-[0.6rem] font-light text-white-snow">{{ this.$t("general." + "Enable" ) }}</p>
                            </div>
                        </div>
                        <div class="w-full border-t border-gray-semimedium"></div>
                    </div>

                    <div v-for="(alert, index) in this.alerts" :key="index"
                        class="flex justify-between items-center rounded-md hover:bg-blue-alerts-hover cursor-pointer group mb-1 gap-6 p-1 pr-2" v-on:click.stop="this.set_Alert_filter(index)"
                        v-bind:class="{ 'hover:bg-blue-alerts-select bg-blue-alerts-select': this.temporal_alert_id == index, 'hidden': this.temporal_alert_deleted.indexOf(index) > -1 }">

                        <div class="flex flex-col justify-between w-full">
                            <a class="inline-flex items-baseline gap-2">
                                <p class="text-white-snow font-light text-regular whitespace-nowrap truncate max-w-[10rem] 2xl:max-w-[12.5rem]">{{ alert.description }}</p>
                            </a>

                            <div class="flex flex-row justify-start whitespace-nowrap">
                                <div v-for="(type_alert, index_ta) in alert.type_alert" :key="index_ta">
                                    <div v-for="(type, index_t) in this.alerts_type" :key="index_t">
                                        <p v-if="type.id == type_alert" class="text-white-snow font-light text-xs whitespace-nowrap truncate max-w-[10rem] 2xl:max-w-[12.5rem]">
                                            {{ (index_ta+1) +'º: '+ this.$t("general." + type.description) + (index_ta != alert.type_alert.length - 1 ? ';&nbsp;' : '') }} </p>
                                    </div>
                                </div>
                            </div>
                        </div>

                        <div class="grid grid-cols-6 place-items-center gap-4 w-full">

                            <i v-on:click.stop="this.set_Alert_filter(index)" class="fa-solid visible" style="font-size:0.9rem" v-bind:style="{ color: !(this.temporal_alert_id == index && this.Edit_Modal == false) ? '#FFFAFA55' : '#FFFAFA' }"
                            v-bind:class="{ 'fa-filter-circle-xmark': this.temporal_alert_id == index && this.Edit_Modal == false, ' fa-filter': !(this.temporal_alert_id == index && this.Edit_Modal == false) }"></i>

                            <i v-on:click.stop="this.open_edit_Modal(index)" class="fa-solid fa-pen" style="font-size:0.9rem; color:#FFFAFAc0;"></i>

                            <i v-on:click.stop="this.open_Timetable(index)" class="fa-solid fa-clock" style="font-size:0.9rem;" v-bind:style="{ color: ((alert.start_at != '00:00' || alert.end_at != '23:59') || alert.enabledDays != JSON.stringify({'0':1,'1':1,'2':1,'3':1,'4':1,'5':1,'6':1})) ? '#FFFAFAc0' : '#FFFAFA55' }"></i>

                            <i class="fa-solid fa-bell" v-bind:class="{'animate-ping' : alert.active == 1}" style="font-size:0.9rem;" v-bind:style="{ color: (alert.active != 1 || alert.enabled == 0) ? '#FFFAFA55' : '#FFFAFA' }"></i>

                            <i v-on:click.stop="this.temporal_alert_id = index; this.open_Delete(false, 'alert')" class="fa-solid fa-trash-can" style="font-size:0.9rem; color:#FFFAFAc0;"></i>

                            <label class="switch w-7" v-on:click.stop="this.switch_AlertEnable(alert.id)">
                                <input v-bind="{ 'checked': alert.enabled == 1 }" :id="('id_switch_' + alert.id)"
                                    type="checkbox">
                                <span class="slider round"></span>
                            </label>
                        </div>
                    </div>

                    <!--<div class="flex justify-end mx-2 p-4 hover:bg-blue-alerts-hover rounded-md"
                        v-on:click="this.open_create_Modal()">
                        <i class="fa-solid fa-plus" style="font-size:1.25rem; color:#FFFAFA;"></i>
                    </div>-->

                </div>

                <!-- END OPEN SIDEBAR ALERTS -->
            </div>
            <!-- END ALERTS SIDEBAR -->
        </div>

        <div v-else style="height: calc(100vh - 4.75rem);" class="bg-darkblue-background z-0 cursor-default select-none flex">

            <div class="relative w-full flex flex-col justify-center items-start pt-2 pb-1 gap-2 px-3" v-on:click="this.Alert_Sidebar_open == true ? this.open_Sidemenu() : '' " >

                <div v-if="this.Alert_Sidebar_open == true" class="absolute inset-0 bg-gray-dark opacity-60"></div>

                <div class="w-full flex flex-row justify-between">
                    <div class="w-full flex flex-col justify-start">
                        <div class="flex flex-row justify-start">
                            <p class="text-white-snow font-light text-2xl text-left">{{ this.$t("general." + "Alerts log") }}</p>
                        </div>
                    </div>
                    <div class="flex justify-end gap-2">
                        <div class="flex items-center justify-center">
                            <Link onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit'); window.history.go(-1)">
                                <button type="button" class="text-white-snow button-blue font-medium rounded-lg text-sm p-1.5 text-center inline-flex items-center">
                                    <i class="fa-solid fa-arrow-left" style="font-size:0.75rem; color:#FFFAFA;"></i>
                                </button>
                            </Link>
                        </div>
                        <div class="flex items-center justify-center">
                            <button type="button" class="text-white-snow button-blue font-medium rounded-lg text-sm text-center flex items-center justify-center w-[1.625rem] h-[1.625rem]" v-on:click.stop="this.open_Sidemenu()"> <!--w-[16rem]-->
                                <i class="fa-solid fa-bell" style="font-size:0.75rem; color:#FFFAFA;"></i>
                            </button>
                        </div>
                    </div>
                </div>

                <div class="bg-blue-searchbar w-full rounded-md shadow-lg border border-gray-semimedium flex items-center" v-bind:class="{ 'flex-row justify-end': !DP_open, 'flex-col justify-center px-2': DP_open }">

                    <input type="text" v-on:keyup="set_SearchFilter()"
                        :placeholder='this.$t("text." + "Search in logs")' id="id_searchbar"
                        class="focus-visible:outline-0 text-md text-left px-4 py-2 text-white-snow font-semilight rounded-l-md bg-transparent w-full">

                    <div class="border-gray-semimedium flex justify-arround items-center" v-bind:class="{ 'border-l pl-2': !DP_open, 'border-t py-0.5 w-full': DP_open }">
                        <Datepicker v-bind:class="{ 'w-12': !DP_open, 'w-full': DP_open }" id="interval_date"
                            v-model="date" :presetRanges="this.presetRanges" vertical range
                            @update:modelValue="set_FilterDates" :clearable="false" :format="this.Format"></Datepicker>
                        <i v-if="DP_open" v-on:click="this.delete_FilterDates()" class="fa-solid fa-xmark mr-4"
                            style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                    </div>

                </div>

                <div class="flex flex-col justify-start gap-y-[1px] bg-darkblue-opa-container rounded-md border border-gray-semimedium w-full" :style="this.getTableHeight()">

                    <div class="py-1 flex justify-start items-center text-gray-light font-light text-center gap-x-3 border-transparent border-l-2 h-8">

                        <div class="flex flex-row items-center justify-start gap-x-6">
                            <div class="flex gap-x-1 rounded-md active:bg-white-hover focus:bg-white-hover hover:bg-white-hover p-2 cursor-pointer">
                                <input type="checkbox" id="checkbox_all"
                                    v-on:click="this.set_CheckboxClick('checkbox_all', 'All_box')">
                                <JetDropdown align="left">
                                    <template #trigger>
                                        <div class="flex flex-row items-center justify-start px-0.5">
                                            <i class="fa-solid fa-caret-down"
                                                style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                                        </div>
                                    </template>
                                    <template #content>
                                        <option v-for="(selector, index) in this.group_selector" :key="index"
                                            :value="selector" class="hover:bg-cyan-opa-2"
                                            v-on:click="this.set_CheckboxClick('checkbox_all', selector)">
                                            {{ selector }}
                                        </option>
                                    </template>
                                </JetDropdown>
                            </div>
                            <i v-if="this.Global_Checkbox_State" v-on:click="this.back_Checkboxes()"
                                class="fa-solid fa-xmark cursor-pointer"
                                style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                            <i v-if="this.Global_Checkbox_State || this.Checkboxes_State"
                                v-on:click="this.set_ReadSelectedAlertLogs(false)"
                                class="fa-solid fa-eye-slash cursor-pointer"
                                style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                            <i v-if="this.Global_Checkbox_State || this.Checkboxes_State"
                                v-on:click="this.set_ReadSelectedAlertLogs(true)" class="fa-solid fa-eye cursor-pointer"
                                style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                            <i v-if="this.Global_Checkbox_State || this.Checkboxes_State"
                                v-on:click="this.open_Delete(false, 'group_alert_log')"
                                class="fa-solid fa-trash-can cursor-pointer"
                                style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                        </div>

                    </div>

                    <div class="overflow-y-auto overflow-x-hidden gap-y-[1px] flex flex-col">
                        <div v-for="(log, index) in this.alerts_log" :key="index"
                            class="hover:border-white border-transparent border-l-2 pl-2 py-1 text-gray-light hover:text-white-snow hover:shadow-lg text-center flex justify-between items-center h-fit cursor-pointer"
                            v-bind:class="{ 'bg-blue-unread hover:bg-blue-unread-hover font-[500]': log.read == 0, 'font-light bg-blue-read hover:bg-blue-read-hover': log.read == 1, 'hidden': temporal_display_logs[index] != true }">

                            <div class="flex justify-start items-center w-fit">
                                <input type="checkbox" :id="'checkbox_' + log.id"
                                    v-on:click="this.set_CheckboxClick(index)">
                            </div>

                            <div v-on:click="this.set_ReadAlertLog(index)"
                                class="flex flex-row-reverse items-center justify-between w-full h-full pl-4 gap-4">

                                <div class="flex flex-col justify-around items-end h-full w-fit pr-3">
                                    <div class="flex flex-row justify-end">
                                        <p class="text-white-snow font-light text-sm text-right">{{ this.formatDate(log.date, 0, false) }}</p>
                                    </div>
                                    <div class="flex justify-end items-end w-fit" v-on:click="this.open_Delete(false, 'alert_log', index)">
                                        <i class="fa-solid fa-trash-can" style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                                    </div>
                                </div>

                                <div class="flex flex-col justify-center h-full w-full">
                                    <div class="flex flex-row justify-start">
                                        <p class="text-white-snow font-light text-left text-md whitespace-nowrap text-ellipsis overflow-hidden" style="max-width: calc(100vw - 10rem);">{{ log.description }}</p>
                                    </div>



                                    <div v-for="(type, index_t) in this.alerts_type" :key="index_t">
                                        <p class="text-white-snow font-light text-sm text-left" v-if="type.id == log.type_alert">{{ this.$t("general." + type.description) }}</p>
                                    </div>
                                </div>

                            </div>
                        </div>
                    </div>
                </div>
            </div>

            <div id="id_Alert_Sidebar" class="fixed right-0 py-4 flex flex-col justify-start overflow-y-scroll overflow-x-hidden h-full border-r-0 border-gray-semimedium bg-darkblue-opa-container-2 duration-200 fload-left"
                v-bind:class="{'w-0 border-0' : !this.Alert_Sidebar_open, 'w-[18rem] border-l shadow-lg' : this.Alert_Sidebar_open}">

                <div>
                    <div class="w-full flex flex-row justify-between px-4 pb-2 mb-2">
                        <div class="w-full flex flex-col justify-start">
                            <div class="flex flex-row justify-start">
                                <p class="text-white-snow font-light text-2xl text-left">{{ this.$t("general." + "Alerts") }}</p>
                            </div>
                        </div>
                        <div class="flex justify-end gap-2">
                            <div class="flex items-center justify-center">
                                <button type="button" class="text-white-snow button-blue font-medium rounded-lg text-sm text-center flex items-center justify-center w-[1.625rem] h-[1.625rem]" v-on:click="this.open_Sidemenu()"> <!--w-[16rem]-->
                                    <i class="fa-solid fa-bell-slash" style="font-size:0.75rem; color:#FFFAFA;"></i>
                                </button>
                            </div>
                        </div>
                    </div>

                    <div class="border-t border-gray-semimedium mx-2 pb-2"></div>

                    <div v-for="(alert, index) in this.alerts" :key="index"
                        class="px-2 mx-2 pt-1 pb-2 flex justify-between rounded-md hover:bg-blue-alerts-hover cursor-pointer group"
                        v-on:click="this.open_edit_Modal(index)"
                        v-bind:class="{ 'hover:bg-blue-alerts-select bg-blue-alerts-select': this.temporal_alert_id == index, 'hidden': this.temporal_alert_deleted.indexOf(index) > -1 }">
                        <!--this.set_Alert_filter(index)"-->

                        <div class="flex flex-col justify-between w-full">
                            <a class="inline-flex items-baseline gap-2">

                                <p class="text-white-snow font-light text-regular whitespace-nowrap text-ellipsis overflow-hidden" style="max-width: 12rem;">{{ alert.description }}</p>

                                <!--<i v-on:click="this.open_edit_Modal(index)"
                                            v-bind:class="{'fa-solid fa-pen': this.temporal_alert_id == index}" style="font-size:0.75rem; color:#FFFAFA;"></i>-->

                                <!--<i v-on:click.stop="this.set_Alert_filter(index)" class="fa-solid group-hover:visible"
                                    style="font-size:0.75rem; color:#FFFAFA;"
                                    v-bind:class="{ 'fa-filter-circle-xmark visible': this.temporal_alert_id == index, ' fa-filter invisible': this.temporal_alert_id != index }"></i>-->

                            </a>

                            <div v-for="(type, index_t) in this.alerts_type" :key="index_t">
                                <p v-if="type.id == alert.type_alert" class="text-white-snow font-light text-xs">
                                    {{ this.$t("general." + type.description) }} </p>
                            </div>
                        </div>

                        <div class="flex justify-between items-center gap-2">
                            <div v-bind:class="{ 'invisible': alert.active != 1 || alert.enabled == 0 }"
                                class="animate-ping">
                                <p class="text-white-snow text-xl font-bold">!</p>
                            </div>
                            <label class="switch w-7" v-on:click.stop="this.switch_AlertEnable(alert.id)">
                                <input v-bind="{ 'checked': alert.enabled == 1 }" :id="('id_switch_' + alert.id)"
                                    type="checkbox">
                                <!-- <span class="slider round"></span> -->
                            </label>
                        </div>

                    </div>

                    <div class="flex justify-end mx-2 p-4 hover:bg-blue-alerts-hover rounded-md"
                        v-on:click="this.open_create_Modal()">
                        <i class="fa-solid fa-plus" style="font-size:1.25rem; color:#FFFAFA;"></i>
                    </div>

                </div>

                <!-- END OPEN SIDEBAR ALERTS -->
            </div>

        </div>

        <!-- START EDIT ALERTS MODAL -->
        <JetModal :show="this.Edit_Modal" :closeable="false">
            <div class="overflow-y-auto max-h-[85vh] pt-3 cursor-default max-w-[40rem]" v-bind:class="{'w-[40rem]' : this.screen_width >= 768, 'w-[90vw]' : this.screen_width < 768}">

                <p class="font-light text-center text-white-snow text-xl">{{ this.$t("general." + "Edit alert") }}</p>

                <div class="grid grid-cols-2 gap-x-4 px-4 pt-4 gap-y-4 font-light text-gray-light text-center content-center items-center">

                    <!-- DESC -->
                    <div v-bind:class="{'col-span-2' : this.screen_width <= 768}">
                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Description") }}</p>
                        <DK_Input id="id_alert_description" type="text" :value="this.alerts[this.temporal_alert_id].description" :border="true" width="full"></DK_Input>
                    </div>

                    <!-- EMAIL -->
                    <div v-bind:class="{'col-span-2' : this.screen_width <= 768}">
                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Email") }}</p>
                        <DK_Input id="id_alert_email" type="email" :value="this.alerts[this.temporal_alert_id].email" :border="true" width="full"></DK_Input>
                    </div>

                    <!-- DEVICES -->
                    <div class="flex flex-col justify-center items-center gap-1"
                        v-bind:class="{ 'col-span-2': (this.checkDeviceType(vmodel_Devices, 'MODBUS') || this.checkDeviceType(vmodel_Devices, 'PULSES')) || this.screen_width <= 768 }">

                        <p class="text-cyan-text text-left ml-1 w-full">{{ this.$t("general." + "Gateways") }}</p>

                        <VueMultiselect :max-height="250" id="id_VueMultiselect_device" v-model="vmodel_Devices"
                            :options="options_Devices" :close-on-select="true" :multiple="false"
                            placeholder='Select gateway' label="name" track-by="name" :allow-empty="false"
                            @select="set_options_Device" />

                    </div>

                    <!--<div class="border-t border-gray-medium w-full col-span-2 m-4"></div>-->

                    <!-- ALARM CONDITIONS -->
                    <!--<div v-bind:class="{'hidden' : this.arr_conditions.length <= 1}" class="flex flex-col gap-1">
                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Alarm conditions")+":" }}</p>
                        <div class="flex justify-evenly items-center gap-2 rounded-md border border-gray-semimedium py-[1px]">
                            <p class="font-normal text-gray-light">{{ this.$t("general.Any condition")}}</p>
                            <label class="switch w-7">
                                <input v-bind="{ 'checked': true }" id='id_conditional_selector' type="checkbox">
                                <span class="slider round"></span>
                            </label>
                            <p class="font-normal text-gray-light">{{ this.$t("general.All conditions")}}</p>
                        </div>
                    </div>-->

                    <!-- Sensor selector -->
                    <div v-for="(condition, index_c) in this.arr_conditions" :key="index_c" class="relative col-span-2 grid grid-cols-2 gap-x-4 px-4 py-4 gap-y-4 font-light text-gray-light text-center content-center items-center rounded-md border border-gray-semimedium">

                        <div class="col-span-2 flex flex-row justify-start items-center gap-x-4" v-on:click="this.ShowCondition(index_c)">
                            <i class="fa-solid transition duration-100 fa-angle-down" v-bind:class="{'-rotate-90 ' : !condition, }" style="font-size:1rem; color:rgba(255,255,255,0.8);"></i>
                            <p class="font-light text-left text-white-snow text-lg">{{ this.$t("general." + "Condition") + ' nº' + (index_c+1)}}</p>
                        </div>

                        <div class="absolute top-0 right-0 p-4 flex justify-end items-center w-fit" v-on:click="this.open_Delete(false, 'condition', index_c)" v-if="this.arr_conditions.length >= 2">
                            <i class="fa-solid fa-trash-can" style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                        </div>

                        <div v-bind:class="{'hidden' : !condition}" class="col-span-2 grid grid-cols-2 gap-x-4 gap-y-4 text-center content-center items-center">

                            <!-- Sensors inside device -->
                            <div class="flex flex-col justify-center items-center gap-1"
                                v-bind:class="{ 'hidden': !vmodel_Devices, 'col-span-2' : this.screen_width <= 768 }"
                                v-if="this.checkDeviceType(vmodel_Devices, 'MODBUS')">

                                <p class="text-cyan-text text-left ml-1 w-full">{{ this.$t("general." + "Sensors") }}</p>

                                <VueMultiselect :max-height="250" :id='"id_VueMultiselect_sensor"+index_c' v-model="vmodel_Sensors[index_c]"
                                    :options="options_Sensors" :close-on-select="true" :multiple="false"
                                    placeholder='Select sensor' label="name" track-by="name" :allow-empty="false"/>

                            </div>

                            <!-- Data_Types -->
                            <div class="flex flex-col justify-center items-center gap-1"
                                v-else-if="this.checkDeviceType(vmodel_Devices, 'PULSES')"
                                v-bind:class="{ 'hidden': !vmodel_Devices, 'col-span-2' : this.screen_width <= 768 }">

                                <p class="text-cyan-text text-left ml-1 w-full">{{ this.$t("general." + "Magnitude") }}</p>

                                <VueMultiselect :max-height="250" :id='"id_VueMultiselect_type"+index_c' v-model="vmodel_Data_Type[index_c]"
                                    :options="options_Data_Type" :close-on-select="true" :multiple="false"
                                    placeholder='Select magnitude' label="description" track-by="description" :allow-empty="false" />

                            </div>

                            <!-- Types -->
                            <div class="flex flex-col justify-center items-center gap-1" v-bind:class="{'col-span-2' : this.screen_width <= 768}">

                                <p class="text-cyan-text text-left ml-1 w-full">{{ this.$t("general." + "Alert type") }}</p>

                                <VueMultiselect :max-height="250" :id='"id_VueMultiselect_type"+index_c' v-model="vmodel_Types[index_c]"
                                    :options="options_Types" :close-on-select="true" :multiple="false"
                                    placeholder='Select type' label="name" track-by="name" :allow-empty="false" />

                            </div>

                            <!-- ALERT SET -->
                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-if="vmodel_Sensors[index_c] && vmodel_Types[index_c] && this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 1)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." + "Threshold") }}</p>

                                <div>
                                    <div class="flex justify-between items-center">

                                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Lower") + ((vmodel_Sensors[index_c].unit && vmodel_Sensors[index_c].unit != '') ? ' (' + vmodel_Sensors[index_c].unit + ')' : '') }}</p>

                                    </div>

                                    <div class="flex justify-between items-center gap-2">

                                        <DK_Input :id='"id_alert_limit_inf_"+index_c' type="text" :value="this.temporal_alert_limit_inf[index_c]" :border="true" width="full" @input="this.enableButton()"></DK_Input>

                                    </div>

                                </div>

                                <div>
                                    <div class="flex justify-between items-center">

                                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Upper") + ((vmodel_Sensors[index_c].unit && vmodel_Sensors[index_c].unit != '') ? ' (' + vmodel_Sensors[index_c].unit + ')' : '') }}</p>

                                    </div>

                                    <div class="flex justify-between items-center gap-2">

                                        <DK_Input :id='"id_alert_limit_sup_"+index_c' type="text" :value="this.temporal_alert_limit_sup[index_c]" :border="true" width="full" @input="this.enableButton()"></DK_Input>

                                    </div>

                                </div>

                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Sensors[index_c] && vmodel_Types[index_c] && this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 2)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "Variation") }}</p>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "It must not fall below the previous measurement by") }}</p>

                                    </div>

                                    <div class="flex justify-center flex-row gap-4">

                                        <div class="flex justify-between items-center gap-2">

                                            <DK_Input :id='"id_alert_limit_inf_"+index_c' type="text" :value="this.temporal_alert_limit_inf[index_c]" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                        </div>

                                        <div class="flex justify-between items-center">

                                            <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">%</p>

                                        </div>

                                    </div>

                                </div>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "It must not exceed the previous measurement by") }}</p>

                                    </div>
                                    <div class="flex justify-center flex-row gap-4">

                                        <div class="flex justify-between items-center gap-2">

                                            <DK_Input :id='"id_alert_limit_sup_"+index_c' type="text" :value="this.temporal_alert_limit_sup[index_c]" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                        </div>

                                        <div class="flex justify-between items-center">

                                            <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">%</p>

                                        </div>
                                    </div>

                                </div>

                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Sensors[index_c] && vmodel_Types[index_c] && this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 3)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "State") }}</p>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "the value has to be equal to") }}</p>

                                    </div>

                                    <div class="flex justify-center flex-row gap-2">

                                        <div class="flex justify-between items-center gap-2">

                                            <DK_Input :id='"id_alert_limit_inf_"+index_c' type="text" :value="this.temporal_alert_limit_inf[index_c]" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                        </div>

                                        <div class="flex justify-between items-center">

                                            <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ ((vmodel_Sensors[index_c].unit && vmodel_Sensors[index_c].unit != '') ? ' (' + vmodel_Sensors[index_c].unit + ')' : '')}}</p>

                                        </div>

                                    </div>

                                </div>

                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Sensors[index_c] && vmodel_Types[index_c] && this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 4)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "Difference") }}</p>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "the value has to be different from") }}</p>

                                    </div>

                                    <div class="flex justify-center flex-row gap-2">

                                        <div class="flex justify-between items-center gap-2">

                                            <DK_Input :id='"id_alert_limit_inf_"+index_c' type="text" :value="this.temporal_alert_limit_inf[index_c]" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                        </div>

                                        <div class="flex justify-between items-center">

                                            <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ ((vmodel_Sensors[index_c].unit && vmodel_Sensors[index_c].unit != '') ? ' (' + vmodel_Sensors[index_c].unit + ')' : '')}}</p>

                                        </div>

                                    </div>

                                </div>

                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Data_Type[index_c] && vmodel_Types[index_c] && !this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 1)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "Threshold") }}</p>

                                <div>
                                    <div class="flex justify-between items-center">

                                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Lower") }}</p>

                                    </div>

                                    <div class="flex justify-between items-center gap-2">

                                        <DK_Input :id='"id_alert_limit_inf_"+index_c' type="text" :value="this.temporal_alert_limit_inf[index_c]" :border="true" width="full" @input="this.enableButton()"></DK_Input>

                                    </div>

                                </div>

                                <div>
                                    <div class="flex justify-between items-center">

                                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Upper") }}</p>

                                    </div>

                                    <div class="flex justify-between items-center gap-2">

                                        <DK_Input :id='"id_alert_limit_sup_"+index_c' type="text" :value="this.temporal_alert_limit_sup[index_c]" :border="true" width="full" @input="this.enableButton()"></DK_Input>

                                    </div>

                                </div>

                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Data_Type[index_c] && vmodel_Types[index_c] && !this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 2)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "Variation") }}</p>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "It must not fall below the previous measurement by") }}</p>

                                    </div>
                                    <div class="flex justify-center flex-row gap-4">

                                    <div class="flex justify-between items-center gap-2">

                                        <DK_Input :id='"id_alert_limit_inf_"+index_c' type="text" :value="this.temporal_alert_limit_inf[index_c]" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                    </div>

                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">%</p>

                                    </div>
                                    </div>

                                </div>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "It must not exceed the previous measurement by") }}</p>

                                    </div>
                                    <div class="flex justify-center flex-row gap-4">

                                        <div class="flex justify-between items-center gap-2">

                                            <DK_Input :id='"id_alert_limit_sup_"+index_c' type="text" :value="this.temporal_alert_limit_sup[index_c]" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                        </div>

                                        <div class="flex justify-between items-center">

                                            <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">%</p>

                                        </div>
                                    </div>
                                </div>
                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Data_Type[index_c] && vmodel_Types[index_c] && !this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 3)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "State") }}</p>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "the value has to be equal to") }}</p>

                                    </div>

                                    <div class="flex justify-center flex-row gap-2">

                                        <div class="flex justify-between items-center gap-2">

                                            <DK_Input :id='"id_alert_limit_inf_"+index_c' type="text" :value="this.temporal_alert_limit_inf[index_c]" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                        </div>

                                    </div>

                                </div>
                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Data_Type[index_c] && vmodel_Types[index_c] && !this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 4)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "Difference") }}</p>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "the value has to be different from") }}</p>

                                    </div>

                                    <div class="flex justify-center flex-row gap-2">

                                        <div class="flex justify-between items-center gap-2">

                                            <DK_Input :id='"id_alert_limit_inf_"+index_c' type="text" :value="this.temporal_alert_limit_inf[index_c]" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                        </div>

                                    </div>

                                </div>
                            </div>
                        </div>
                    </div>

                    <div v-bind:class="{'col-span-2' : this.screen_width <= 768}" v-if="vmodel_Devices">
                        <p v-on:click="this.AddCondition()" class="w-fit font-light text-left text-gray-light hover:text-white-snow hover:underline">+ Add condition</p>
                    </div>

                </div>

                <!-- BUTTONS -->
                <div class="mt-4 flex col-span-10 gap-2 px-4 pb-4 font-light text-gray-light text-center" v-bind:class="{'justify-end' : this.screen_width >= 768, 'justify-center' : this.screen_width < 768}">

                    <DK_Button color="blue" :fixedSize="true" v-bind:class="{ 'hidden': !this.enableButtonvar }" v-on:click="this.save_EditAlert()" >
                        {{ this.$t("general.Update") }}
                    </DK_Button>

                    <DK_Button color="blue" :fixedSize="true" v-on:click="this.cancel_EditAlert()" >
                        {{ this.$t("general.Cancel") }}
                    </DK_Button>

                    <DK_Button color="red" :fixedSize="true" v-on:click="this.open_Delete(false, 'alert')" >
                        {{ this.$t("general.Delete") }}
                    </DK_Button>

                </div>
            </div>

        </JetModal>
        <!-- END EDIT ALERTS MODAL -->

        <!-- START CREATE ALERTS MODAL -->
        <JetModal :show="this.Create_Modal" :closeable="false">
            <div class="overflow-y-auto max-h-[85vh] pt-3 cursor-default flex flex-col justify-between max-w-[40rem]"  v-bind:class="{'w-[40rem]' : this.screen_width >= 768, 'w-[90vw]' : this.screen_width < 768}">

                <p class="font-light text-center text-white-snow text-xl">{{ this.$t("general." + "Create alert") }}</p>

                <div class="grid grid-cols-2 gap-x-4 px-4 pt-4 gap-y-4 font-light text-gray-light text-center content-center items-center">

                    <!-- DESC -->
                    <div v-bind:class="{'col-span-2' : this.screen_width <= 768}">
                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Description") }}</p>
                        <DK_Input id="id_alert_description" type="text" value="" :border="true" width="full"></DK_Input>

                    </div>

                    <!-- EMAIL -->
                    <div v-bind:class="{'col-span-2' : this.screen_width <= 768}">
                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Email") }}</p>
                        <DK_Input id="id_alert_email" type="email" value="" :border="true" width="full"></DK_Input>

                    </div>

                    <!-- DEVICES -->
                    <div class="flex flex-col justify-center items-center gap-1 col-span-2">

                        <p class="text-cyan-text text-left ml-1 w-full">{{ this.$t("general." + "Gateways") }}</p>

                        <div class="w-full" v-bind:class="{'h-[250px] ' : this.isOpen_create && !vmodel_Devices}">
                            <VueMultiselect :max-height="250" id="id_VueMultiselect_device" v-model="vmodel_Devices"
                                :options="options_Devices" :close-on-select="true" :multiple="false" label="name"
                                placeholder='Select gateway' track-by="name" :allow-empty="false" @select="set_options_Device"
                                :isOpen="isOpen_create" @open="isOpen_create = true" @close="isOpen_create = false"/>
                        </div>

                    </div>

                    <!--<div class="border-t border-gray-medium w-full col-span-2 m-4"></div>-->

                    <!-- ALARM CONDITIONS -->
                    <!--<div v-bind:class="{'hidden' : this.arr_conditions.length <= 1}">
                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Alarm conditions")+":" }}</p>
                        <div class="flex justify-evenly items-center gap-2 rounded-md border border-gray-semimedium py-[1px]">
                            <p class="font-normal text-gray-light">{{ this.$t("general.Any condition")}}</p>
                            <label class="switch w-7">
                                <input v-bind="{ 'checked': true }" id='id_conditional_selector' type="checkbox">
                                <span class="slider round"></span>
                            </label>
                            <p class="font-normal text-gray-light">{{ this.$t("general.All conditions")}}</p>
                        </div>
                    </div>-->

                    <!-- Sensor selector -->
                    <div v-for="(condition, index_c) in this.arr_conditions" :key="index_c" class="relative col-span-2 grid grid-cols-2 gap-x-4 px-4 py-4 gap-y-4 font-light text-gray-light text-center content-center items-center rounded-md border border-gray-semimedium"
                        v-bind:class="{'hidden' : !vmodel_Devices}">

                        <div class="col-span-2 flex flex-row justify-start items-center gap-x-4" v-on:click="this.ShowCondition(index_c)">
                            <i class="fa-solid transition duration-100 fa-angle-down" v-bind:class="{'-rotate-90 ' : !condition, }" style="font-size:1rem; color:rgba(255,255,255,0.8);"></i>
                            <p class="font-light text-left text-white-snow text-lg">{{ this.$t("general." + "Condition") + ' nº' + (index_c+1)}}</p>
                        </div>

                        <div class="absolute top-0 right-0 p-4 flex justify-end items-center w-fit" v-on:click="this.open_Delete(false, 'condition', index_c)" v-if="this.arr_conditions.length >= 2">
                            <i class="fa-solid fa-trash-can" style="font-size:0.75rem; color:rgba(209, 209, 209, 0.8);"></i>
                        </div>

                        <div v-bind:class="{'hidden' : !condition}" class="col-span-2 grid grid-cols-2 gap-x-4 gap-y-4 text-center content-center items-center">

                            <!-- Sensors inside device -->
                            <div class="flex flex-col justify-center items-center gap-1"
                                v-bind:class="{ 'col-span-2' : this.screen_width <= 768 }"
                                v-if="this.checkDeviceType(vmodel_Devices, 'MODBUS') && vmodel_Devices">

                                <p class="text-cyan-text text-left ml-1 w-full">{{ this.$t("general." + "Sensors") }}</p>

                                <VueMultiselect :max-height="250" id="id_VueMultiselect_sensor" v-model="vmodel_Sensors[index_c]"
                                    :options="options_Sensors" :close-on-select="true" :multiple="false"
                                    placeholder='Select sensor' label="name" track-by="name" :allow-empty="false" />

                            </div>

                            <!-- Data_Types -->
                            <div class="flex flex-col justify-center items-center gap-1"
                                v-else-if="this.checkDeviceType(vmodel_Devices, 'PULSES') && vmodel_Devices"
                                v-bind:class="{'col-span-2' : this.screen_width <= 768 }">

                                <p class="text-cyan-text text-left ml-1 w-full">{{ this.$t("general." + "Magnitude") }}</p>

                                <VueMultiselect :max-height="250" id="id_VueMultiselect_type" v-model="vmodel_Data_Type[index_c]"
                                    :options="options_Data_Type" :close-on-select="true" :multiple="false"
                                    placeholder='Select magnitude' label="description" track-by="description" :allow-empty="false" />

                            </div>

                            <!-- Types -->
                            <div v-if="vmodel_Devices" class="flex flex-col justify-center items-center gap-1" v-bind:class="{'col-span-2' : this.screen_width <= 768}">

                                <p class="text-cyan-text text-left ml-1 w-full">{{ this.$t("general." + "Alert type") }}</p>

                                <VueMultiselect :max-height="250" id="id_VueMultiselect_type" v-model="vmodel_Types[index_c]"
                                    :options="options_Types" :close-on-select="true" :multiple="false"
                                    placeholder='Select type' label="name" track-by="name" :allow-empty="false" />

                            </div>

                            <!-- ALERT SET -->

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-if="vmodel_Sensors[index_c] && vmodel_Types[index_c] && this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 1)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "Threshold") }}</p>

                                <div>
                                    <div class="flex justify-between items-center">

                                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Lower") + ((vmodel_Sensors[index_c].unit && vmodel_Sensors[index_c].unit != '') ? ' (' + vmodel_Sensors[index_c].unit + ')' : '') }}</p>

                                    </div>

                                    <div class="flex justify-between items-center gap-2">

                                        <DK_Input :id="'id_alert_limit_inf_'+index_c" type="text" value="" :border="true" width="full" @input="this.enableButton()"></DK_Input>

                                    </div>

                                </div>

                                <div>
                                    <div class="flex justify-between items-center">

                                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Upper") + ((vmodel_Sensors[index_c].unit && vmodel_Sensors[index_c].unit != '') ? ' (' + vmodel_Sensors[index_c].unit + ')' : '') }}</p>

                                    </div>

                                    <div class="flex justify-between items-center gap-2">

                                        <DK_Input :id="'id_alert_limit_sup_'+index_c" type="text" value="" :border="true" width="full" @input="this.enableButton()"></DK_Input>

                                    </div>

                                </div>

                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Sensors[index_c] && vmodel_Types[index_c] && this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 2)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "Variation") }}</p>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "It must not fall below the previous measurement by") }}</p>

                                    </div>
                                    <div class="flex justify-center flex-row gap-4">

                                    <div class="flex justify-between items-center gap-2">

                                        <DK_Input :id="'id_alert_limit_inf_'+index_c" type="text" value="" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                    </div>

                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">%</p>

                                    </div>
                                    </div>

                                </div>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "It must not exceed the previous measurement by") }}</p>

                                    </div>
                                    <div class="flex justify-center flex-row gap-4">

                                    <div class="flex justify-between items-center gap-2">

                                        <DK_Input :id="'id_alert_limit_sup_'+index_c" type="text" value="" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                    </div>

                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">%</p>

                                    </div>
                                    </div>

                                </div>

                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Sensors[index_c] && vmodel_Types[index_c] && this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 3)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "State") }}</p>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "the value has to be equal to") }}</p>

                                    </div>

                                    <div class="flex justify-center flex-row gap-2">

                                        <div class="flex justify-between items-center gap-2">

                                            <DK_Input :id="'id_alert_limit_inf_'+index_c" type="text" value="" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                        </div>

                                        <div class="flex justify-between items-center">

                                            <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ ((vmodel_Sensors[index_c].unit && vmodel_Sensors[index_c].unit != '') ? ' (' + vmodel_Sensors[index_c].unit + ')' : '')}}</p>

                                        </div>

                                    </div>
                                </div>
                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Sensors[index_c] && vmodel_Types[index_c] && this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 4)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "Difference") }}</p>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "the value has to be different from") }}</p>

                                    </div>

                                    <div class="flex justify-center flex-row gap-2">

                                        <div class="flex justify-between items-center gap-2">

                                            <DK_Input :id="'id_alert_limit_inf_'+index_c" type="text" value="" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                        </div>

                                        <div class="flex justify-between items-center">

                                            <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ ((vmodel_Sensors[index_c].unit && vmodel_Sensors[index_c].unit != '') ? ' (' + vmodel_Sensors[index_c].unit + ')' : '')}}</p>

                                        </div>

                                    </div>
                                </div>
                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Data_Type[index_c] && vmodel_Types[index_c] && !this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 1)">


                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "Threshold") }}</p>

                                <div>
                                    <div class="flex justify-between items-center">

                                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Lower") }}</p>

                                    </div>

                                    <div class="flex justify-between items-center gap-2">

                                        <DK_Input :id="'id_alert_limit_inf_'+index_c" type="text" value="" :border="true" width="full" @input="this.enableButton()"></DK_Input>

                                    </div>

                                </div>

                                <div>
                                    <div class="flex justify-between items-center">

                                        <p class="text-cyan-text text-left ml-1">{{ this.$t("general." + "Upper") }}</p>

                                    </div>

                                    <div class="flex justify-between items-center gap-2">

                                        <DK_Input :id="'id_alert_limit_sup_'+index_c" type="text" value="" :border="true" width="full" @input="this.enableButton()"></DK_Input>

                                    </div>

                                </div>

                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Data_Type[index_c] && vmodel_Types[index_c] && !this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 2)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "Variation") }}</p>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "It must not fall below the previous measurement by") }}</p>

                                    </div>
                                    <div class="flex justify-center flex-row gap-4">

                                        <div class="flex justify-between items-center gap-2">

                                            <DK_Input :id="'id_alert_limit_inf_'+index_c" type="text" value="" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                        </div>

                                        <div class="flex justify-between items-center">

                                            <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">%</p>

                                        </div>
                                    </div>

                                </div>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "It must not exceed the previous measurement by") }}</p>

                                    </div>
                                    <div class="flex justify-center flex-row gap-4">

                                        <div class="flex justify-between items-center gap-2">

                                            <DK_Input :id="'id_alert_limit_sup_'+index_c" type="text" value="" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                        </div>

                                        <div class="flex justify-between items-center">

                                            <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">%</p>

                                        </div>
                                    </div>

                                </div>

                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Data_Type[index_c] && vmodel_Types[index_c] && !this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 3)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "State") }}</p>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "the value has to be equal to") }}</p>

                                    </div>

                                    <div class="flex justify-center flex-row gap-2">

                                        <div class="flex justify-between items-center gap-2">

                                            <DK_Input :id="'id_alert_limit_inf_'+index_c" type="text" value="" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                        </div>

                                    </div>
                                </div>
                            </div>

                            <div class="col-span-2 baltoro-container grid grid-cols-2 gap-4 p-4" v-else-if="vmodel_Data_Type[index_c] && vmodel_Types[index_c] && !this.checkDeviceType(vmodel_Devices, 'MODBUS') && (vmodel_Types[index_c].id == 4)">

                                <p class="font-light text-center text-white-snow text-xl col-span-2">{{ this.$t("general." +
                                    "Difference") }}</p>

                                <div class="col-span-2 flex w-full justify-center gap-2" v-bind:class="{'flex-col items-center' : this.screen_width <= 768}">
                                    <div class="flex justify-between items-center">

                                        <p class="text-gray-light" v-bind:class="{'text-left' : this.screen_width >= 768}">{{ this.$t("text." + "the value has to be different from") }}</p>

                                    </div>

                                    <div class="flex justify-center flex-row gap-2">

                                        <div class="flex justify-between items-center gap-2">

                                            <DK_Input :id="'id_alert_limit_inf_'+index_c" type="text" value="" :border="true" width="4rem" @input="this.enableButton()"></DK_Input>

                                        </div>

                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>

                    <div v-bind:class="{'col-span-2' : this.screen_width <= 768}" v-if="vmodel_Devices">
                        <p v-on:click="this.AddCondition()" class="w-fit font-light text-left text-gray-light hover:text-white-snow hover:underline">+ Add condition</p>
                    </div>

                </div>

                <!-- BUTTONS -->
                <div class="mt-4 flex col-span-10 gap-2 px-4 pb-4 font-light text-gray-light text-center" v-bind:class="{'justify-end' : this.screen_width >= 768, 'justify-center' : this.screen_width < 768}">


                        <DK_Button color="blue" :fixedSize="true" v-bind:class="{ 'hidden': !this.enableButtonvar }" v-on:click="this.save_CreateAlert()" >
                            {{ this.$t("general.Create") }}
                        </DK_Button>

                        <DK_Button color="blue" :fixedSize="true" v-on:click="this.cancel_CreateAlert()" >
                            {{ this.$t("general.Cancel") }}
                        </DK_Button>

                </div>
            </div>
        </JetModal>
        <!-- END CREATE ALERTS MODAL -->

        <!-- START TIMETABLE EDITION MODAL -->
        <JetModal :show="this.Timetable_Modal" :closeable="false">

            <div class="pt-3 cursor-default flex flex-col justify-between max-w-[40rem] gap-4"  v-bind:class="{'w-[40rem]' : this.screen_width >= 768, 'w-[90vw]' : this.screen_width < 768}">

                <p class="font-light text-center text-white-snow text-xl">{{ this.$t("general." + "Edit timetable") }}</p>

                <!-- <div class="w-full px-10 grid place-items-center h-[15rem]">
                    <vue3-slider class="w-full" :v-model="this.vmodel_Slider"
                    orientation="circular" width="15rem" color="#05fff3" track-color="#FFFAFA55" alwaysShowHandle="true"
                    min="0" max="96" steps="96" :formatTooltip="this.ClockTooltip" flipTooltip="true"
                    tooltip="true" tooltipColor="#FFFAFA55" tooltipTextColor="#FFFAFA"/>
                </div> -->

                <div class="w-full grid grid-cols-2 px-4 gap-8">
                    <div class="flex flex-col justify-start items-start w-full py-2 gap-2">
                        <p class="text-cyan-text text-left">{{ this.$t("general." + "Days active") }}:</p>
                        <div class="flex flex-row justify-evenly items-center w-full gap-1">
                            <div v-for="(day, index) in activeAlarmDays" :key="index" class="rounded-full border text-center w-8 h-8 flex items-center justify-center"
                                v-bind:class="{'bg-cyan-opa-0 border-white': activeAlarmDays[(index+1)%7] == true, 'bg-cyan-opa opacity-50 border-gray-semimedium': activeAlarmDays[(index+1)%7] == false }"
                                v-on:click="this.SwitchDayActive((index+1)%7)">
                                {{ this.$t("general." + this.GetDay((index+1)%7)).charAt(0).toUpperCase() }}
                            </div>
                        </div>
                        <div class="grid grid-cols-2 place-items-center gap-2 w-full px-1 mt-1">

                            <DK_Button color="blue" :fixedSize="false" width="full" v-on:click="this.SwitchDayActive(null, 0)">
                                {{ this.$t("general.Midweek") }}
                            </DK_Button>

                            <DK_Button color="blue" :fixedSize="false" width="full" v-on:click="this.SwitchDayActive(null, 1)">
                                {{ this.$t("general.Week-end") }}
                            </DK_Button>

                        </div>
                    </div>
                    <div class="flex flex-col justify-start items-start w-full py-2 gap-2">
                        <p class="text-cyan-text text-left">{{ this.$t("general." + "Active hours") }}:</p>
                        <div class="flex flex-row justify-start items-center gap-4">

                            <label>{{ this.$t("general.From") }}</label>

                            <div class="flex flex-row items-baseline gap-1">
                                <input type="number" id="hora" v-model="this.start_at.hour" @input="validarHora" min="0" max="23" required>
                                <label>:</label>
                                <input type="number" id="minuto" v-model="this.start_at.min" @input="validarMinuto" min="0" max="59" required>
                            </div>

                            <label>{{ this.$t("general.Into").toLowerCase() }}</label>

                            <div class="flex flex-row items-baseline gap-1">
                                <input type="number" id="hora" v-model="this.end_at.hour" @input="validarHora" min="0" max="23" required>
                                <label>:</label>
                                <input type="number" id="minuto" v-model="this.end_at.min" @input="validarMinuto" min="0" max="59" required>
                            </div>

                        </div>

                        <div class="w-full flex flex-col mt-4">

                            <div class="w-full flex flex-row justify-between items-baseline text-xs">
                                <p>00h00</p>
                                <p>12h00</p>
                                <p>23h59</p>
                            </div>

                            <div class="flex flex-row h-[0.25rem] items-baseline justify-center w-full rounded-full">
                                <div v-for="(quarter_hour, index) in (new Array(24*4))" :key="quarter_hour" class="w-full h-full bg-cyan-text"
                                    v-bind:class="{'bg-cyan-text': this.GetQuarterHourColor(index), 'bg-cyan-opa opacity-50': !this.GetQuarterHourColor(index) }">
                                </div>
                            </div>

                        </div>
                    </div>
                </div>

                <div class="flex col-span-10 gap-2 px-4 pb-4 font-light text-gray-light text-center" v-bind:class="{'justify-end' : this.screen_width >= 768, 'justify-center' : this.screen_width < 768}">

                    <DK_Button color="blue" :fixedSize="true" v-on:click="this.save_Timetable()">
                        {{ this.$t("general.Save") }}
                    </DK_Button>

                    <DK_Button color="blue" :fixedSize="true" v-on:click="this.cancel_Timetable()" >
                        {{ this.$t("general.Cancel") }}
                    </DK_Button>

                </div>

            </div>

        </JetModal>
        <!-- END TIMETABLE EDITION MODAL -->

        <!-- START DELETE CONFIRMATION MODAL -->
        <JetModal :show="this.Delete_Modal" :closeable="false">

            <div class="max-w-[90vw]">

                <p class="mt-3 font-light text-center text-white-snow text-xl">{{ this.$t("text." + "Are you sure?") }}</p>

                <p class="mt-3 font-light text-center text-gray-light px-4 max-w-[90vw]">{{ this.$t("text.Once deleted, there will be no way to retrieve it")}}.</p>

                <div class="flex col-span-10 gap-2 px-4 pb-4 font-light text-gray-light text-center pt-4 justify-center">

                    <DK_Button color="blue" :fixedSize="true" v-on:click="this.close_Delete()">
                        {{ this.$t("general.Keep") }}
                    </DK_Button>

                    <DK_Button color="red" :fixedSize="true" v-on:click="this.open_Delete(true, temporal_selector_delete)" >
                        {{ this.$t("general.Delete") }}
                    </DK_Button>

                </div>
            </div>
        </JetModal>
        <!-- END DELETE CONFIRMATION MODAL -->

    </AppLayout>
</template>

<script>
export default {
    data() {

        return {

            screen_width: 0,
            Alert_Sidebar_open: false,

            midweekEquals: true,
            weekendEquals: true,

            activeAlarmDays: new Array(7).fill(false),
            start_at: {
                hour: 0,
                min: 0,
            },
            end_at: {
                hour: 23,
                min: 59,
            },
            EnableSaveHours: false,

            // Multiselect
            options_Devices: [],
            options_Sensors: [],
            options_Types: [],
            options_Data_Type: [],

            isOpen_create: false,

            vmodel_Devices: undefined,
            vmodel_Sensors: undefined,
            vmodel_Types: undefined,
            vmodel_Data_Type: undefined,
            vmodel_Timetable: undefined,

            // Datepicker
            DP_open: false, // if the DP has selected date

            // Modals
            Edit_Modal: false, // if the Edit Modal is open == true, closed==false;
            Delete_Modal: false, // if the Delete Confirmation Modal is open == true, closed==false;
            Create_Modal: false, // if the Create Modal is open == true, closed==false;
            Timetable_Modal: false, // if the Timetable Modal is open == true, closed==false;

            // Checkboxes
            Global_Checkbox_State: false, // contains the state of the global checkbox
            Checkboxes_State: false, // if any other checkbox is checked == true; else false

            // Temporal global vars
            temporal_selector_delete: null, // contains the item identifier we are deleting
            temporal_alert_deleted: new Array(), //contains the deleted alerts not to show them

            // Temporal alert vars
            temporal_alert_id: null, // contains the id when we select an alert
            arr_conditions: new Array(), // contains the conditions of selected alert

            // Temporal log vars
            temporal_alert_log_id: null, // to set alert log modal
            temporal_selected_logs: new Array(this.alerts_log.length),  // array where we store the state of each log checkbox

            temporal_display_logs: (new Array(this.alerts_log.length)).fill(true),  // array where we store if the log is being filtered (true == display the log; else hidden)
            temporal_display_deleted: (new Array(this.alerts_log.length)).fill(true),  // array where we store if the log should be displayed depending if he was or not deleted

            temporal_display_filter_search: (new Array(this.alerts_log.length)).fill(true),  // array where we store if the log should be displayed depending the searching filter
            temporal_display_filter_dates: (new Array(this.alerts_log.length)).fill(true),  // array where we store if the log should be displayed depending the dates filter
            temporal_display_filter_alerts: (new Array(this.alerts_log.length)).fill(true),  // array where we store if the log should be displayed depending the description alert filter

            // Others
            Delete_index: null,
            date: [this.get_InitialDates('first'), this.get_InitialDates('last')],
            initial_date: new Array(2),
            group_selector: ['All', 'None', 'Read', 'Unread'], // array to filter checks
            enableButtonvar: false,

            presetRanges: [
                { label: this.$t("general." + "Today"), range: [startOfDay(new Date()), new Date()] },
                { label: this.$t("general." + "Yesterday"), range: [startOfDay(subDays(new Date(), 1)), endOfDay(subDays(new Date(), 1))] },
                { label: this.$t("general." + "Last 7 Days"), range: [new Date().setDate(new Date().getDate() - 7), new Date().setDate(new Date().getDate())] },
                { label: this.$t("general." + "Last 14 Days"), range: [new Date().setDate(new Date().getDate() - 14), new Date().setDate(new Date().getDate())] },
                { label: this.$t("general." + "Last 30 Days"), range: [new Date().setDate(new Date().getDate() - 30), new Date().setDate(new Date().getDate())] },
                { label: this.$t("general." + "This month"), range: [startOfMonth(new Date()), new Date()] },
                { label: this.$t("general." + "Last month"), range: [startOfMonth(subMonths(new Date(), 1)), endOfMonth(subMonths(new Date(), 1))] }
            ],

            getTableHeight() {
                return {
                    height: `calc(100vh - ${this.DP_open ? '14.447rem' : '12rem'})`,
                };
            },

            formatDate: (date, utc, short) => {
                date = new Date(date);
                // utc en 0 para formatear hora local del usuario, 1 para formatear UTC
                if (utc == 0) {
                    date.setHours(date.getHours() + this.user.timezone_offset);
                } else if (utc == -1) {
                    date.setHours(date.getHours() - this.user.timezone_offset);
                }

                if(this.screen_width >= 768){

                    if (!short) {
                        return (
                            date.getFullYear() +
                            "-" +
                            ((date.getMonth() + 1 + "").length < 2
                                ? "0" + (date.getMonth() + 1)
                                : date.getMonth() + 1) +
                            "-" +
                            ((date.getDate() + "").length < 2
                                ? "0" + date.getDate()
                                : date.getDate()) +
                            " " +
                            ((date.getHours() + "").length < 2
                                ? "0" + date.getHours()
                                : date.getHours()) +
                            ":" +
                            ((date.getMinutes() + "").length < 2
                                ? "0" + date.getMinutes()
                                : date.getMinutes()) +
                            ":" +
                            ((date.getSeconds() + "").length < 2
                                ? "0" + date.getSeconds()
                                : date.getSeconds())
                        );
                    } else {
                        return (
                            ((date.getMonth() + 1 + "").length < 2
                                ? "0" + (date.getMonth() + 1)
                                : date.getMonth() + 1) +
                            "-" +
                            ((date.getDate() + "").length < 2
                                ? "0" + date.getDate()
                                : date.getDate()));
                    }

                }else{

                    const fechaActual = new Date();

                    if (fechaActual.toDateString() === date.toDateString()) {


                        return (

                            ((date.getHours() + "").length < 2
                                ? "0" + date.getHours()
                                : date.getHours()) +
                            ":" +
                            ((date.getMinutes() + "").length < 2
                                ? "0" + date.getMinutes()
                                : date.getMinutes())
                        );
                    } else {

                        return (
                            date.getFullYear() +
                            "/" +
                            ((date.getMonth() + 1 + "").length < 2
                                ? "0" + (date.getMonth() + 1)
                                : date.getMonth() + 1) +
                            "/" +
                            ((date.getDate() + "").length < 2
                                ? "0" + date.getDate()
                                : date.getDate())

                        );

                    }

                }

            },

        };
    },

    props: ['gateways', 'alerts', 'alerts_log', 'alerts_type', 'sensors', 'data_type', 'user'],

    // gateways contains each device of the group
    // alerts contains each alert configuration
    // alerts_log contains each log configuration, state, etc
    // alerts_type contains the text linked with the alert_type int in alerts and alerts_log
    // user contains the user info like timezone_offset or token

    components: {
        // "vue3-slider": slider
    },

    beforeCreate() {
        this.$i18n.locale = this.user.language
    },

    beforeMount(){
        this.screen_width = window.screen.width;
    },

    mounted() {

        window.emitter.emit('changeLoaderStatus', false, 'orbit')

        this.setAlertsTextNavbar()

    },

    methods: {

        open_Sidemenu(){
            if(this.Alert_Sidebar_open == false){
                this.Alert_Sidebar_open = true;
            }else{
                this.Alert_Sidebar_open = false;
            }
        },

        checkDeviceType(selected_gateway, type) {

            let bool = false;

            this.gateways.forEach((gateway, index) => {
                if(selected_gateway){
                    if (gateway.id == selected_gateway.id) {
                        if (
                            gateway.type_name.indexOf(type) != -1 ||
                            (gateway.MODBUS_device_type != "" && type == "MODBUS") ||
                            gateway.solutions_id == 9
                        ) {
                            bool = true;
                        }
                    }
                }
            })

            return bool;
        },

        enableButton() {

            let visibility = true;
            var numsFilter = /^-?\d+(\.\d+)?$/;

            if (document.getElementById('id_alert_description')) {
                if ((document.getElementById('id_alert_description').value.trim().length === 0) && visibility == true) {
                    visibility = false;
                }
            } else {
                visibility = false;
            }

            if (document.getElementById('id_alert_email')) {
                if ((document.getElementById('id_alert_email').value.trim().length === 0) && visibility == true) {
                    visibility = false;
                }
            } else {
                visibility = false;
            }

            this.arr_conditions.forEach( (condition, index) => {

                if (document.getElementById('id_alert_limit_sup_'+index) || document.getElementById('id_alert_limit_inf_'+index)) {
                    if (
                        (document.getElementById('id_alert_limit_sup_'+index) && document.getElementById('id_alert_limit_sup_'+index).value.trim().length === 0) &&
                        (document.getElementById('id_alert_limit_inf_'+index) && document.getElementById('id_alert_limit_inf_'+index).value.trim().length === 0) &&
                        visibility == true
                    ) {
                        visibility = false;
                    }

                    if(
                        (document.getElementById('id_alert_limit_sup_'+index) && !numsFilter.test(document.getElementById('id_alert_limit_sup_'+index).value)) &&
                        (document.getElementById('id_alert_limit_inf_'+index) && !numsFilter.test(document.getElementById('id_alert_limit_inf_'+index).value)) &&
                        visibility == true
                    ){
                        visibility = false;
                    }

                } else {
                    visibility = false;
                }

            })

            this.enableButtonvar = visibility;

        },

        unset_limits: function (item) {

            document.getElementById('id_alert_limit_inf').value = '';
            document.getElementById('id_alert_limit_sup').value = '';
            this.arr_conditions = new Array();

        },

        set_options_Device: function (item) {

            this.arr_conditions = new Array(1).fill(true);

            this.vmodel_Sensors = new Array(this.arr_conditions.length)
            this.vmodel_Data_Type = new Array(this.arr_conditions.length)
            this.vmodel_Types = new Array(this.arr_conditions.length)

            this.options_Sensors = [];
            this.options_Data_Type = [];

            /* this.vmodel_Types = new Array(this.arr_conditions.length) */

            if(this.checkDeviceType(item, 'MODBUS')){

                this.sensors.forEach((sensor, key) => {

                    if (sensor.id == item.id) {
                        for (let key in sensor.names) {
                            let obj = {
                                name: sensor.names[key],
                                unit: sensor.units[key],
                                id: key,
                            }
                            this.options_Sensors.push(obj)
                        }
                    }
                })

            } else if(this.checkDeviceType(item, 'PULSES')){

                this.data_type.forEach((type, key) => {

                    let obj = {
                        description: type.description,
                        frame_type: type.frame_type,
                        id: type.id,
                    }
                    this.options_Data_Type.push(obj)
                })
            }
        },

        // opens modal to edit this.alerts[index]
        open_edit_Modal(index) {

            this.Edit_Modal = true;
            this.temporal_alert_id = index;

            this.options_Devices = [];
            this.gateways.forEach((value, index_g) => {
                //if(value.solutions_id != null){
                    let obj = {
                        name: value.name, //+ ' (' + value.id + ')',
                        id: value.id
                    }
                    this.options_Devices.push(obj)

                    if (this.alerts[index].iddevice == value.id) {
                        this.vmodel_Devices = obj
                    }
                //}
            })

            this.arr_conditions = new Array(
                this.checkDeviceType(this.vmodel_Devices, 'MODBUS') ? this.alerts[index].sensor_key.length : this.alerts[index].type_alert.length
            ).fill(false);

            this.vmodel_Sensors = new Array(this.arr_conditions.length)
            this.vmodel_Data_Type = new Array(this.arr_conditions.length)
            this.vmodel_Types = new Array(this.arr_conditions.length)

            this.options_Sensors = [];
            this.options_Data_Type = [];

            if(this.checkDeviceType(this.vmodel_Devices, 'MODBUS')){

                if(this.sensors.length > 1){
                    this.sensors.forEach((sensor, key) => {

                        if (sensor.id == this.vmodel_Devices.id) {
                            for (let key in sensor.names) {
                                let obj = {
                                    name: sensor.names[key],
                                    unit: sensor.units[key],
                                    id: key,
                                }
                                this.options_Sensors.push(obj)

                                this.arr_conditions.forEach( (condition, index_c) => {
                                    if (this.alerts[index].sensor_key[index_c] == key) {
                                        this.vmodel_Sensors[index_c] = obj
                                    }
                                })
                            }
                        }
                    })

                } else {

                    let gateway = this.sensors[Object.keys(this.sensors).find(key => this.sensors[key] !== undefined)];

                    if (gateway.id == this.vmodel_Devices.id) {
                        for (let key in gateway.names) {
                            let obj = {
                                name: gateway.names[key],
                                unit: gateway.units[key],
                                id: key,
                            }
                            this.options_Sensors.push(obj)

                            this.arr_conditions.forEach( (condition, index_c) => {
                                if (this.alerts[index].sensor_key[index_c] == key) {
                                    this.vmodel_Sensors[index_c] = obj
                                }
                            })
                        }
                    }
                }

            } else if(this.checkDeviceType(this.vmodel_Devices, 'PULSES')){

                this.data_type.forEach((type, key) => {

                    let obj = {
                        description: type.description,
                        frame_type: type.frame_type,
                        id: type.id,
                    }

                    this.options_Data_Type.push(obj)

                    this.arr_conditions.forEach( (condition, index_c) => {
                        if (this.alerts[index].iddata_type[index_c] == obj.id) {
                            this.vmodel_Data_Type[index_c] = obj
                        }
                    })

                })

            }

            this.options_Types = [];

            this.alerts_type.forEach((type, key) => {

                let obj = {
                    name: type.description,
                    id: type.id,
                }

                this.options_Types.push(obj)

                this.arr_conditions.forEach( (condition, index_c) => {
                    if (this.alerts[index].type_alert[index_c] == type.id) {
                        this.vmodel_Types[index_c] = obj;
                    }
                })
            })

            this.temporal_alert_limit_inf = [...this.alerts[this.temporal_alert_id].limit_inf]
            this.temporal_alert_limit_sup = [...this.alerts[this.temporal_alert_id].limit_sup]

            setTimeout(() => {
                this.enableButton()
            }, 0);

        },

        save_EditAlert() {

            let limit_inf = new Array();
            let limit_sup = new Array();

            let sensor_key = '';
            let type_alert = '';
            let iddata_type = '';

            let limit_inf_str = '';
            let limit_sup_str = '';

            this.arr_conditions.forEach((condition, index) => {

                limit_inf.push(document.getElementById('id_alert_limit_inf_'+index) ? (document.getElementById('id_alert_limit_inf_'+index).value == '' ? null : document.getElementById('id_alert_limit_inf_'+index).value) : null)
                limit_sup.push(document.getElementById('id_alert_limit_sup_'+index) ? (document.getElementById('id_alert_limit_sup_'+index).value == '' ? null : document.getElementById('id_alert_limit_sup_'+index).value) : null)

                iddata_type += this.vmodel_Data_Type[index] ? (this.vmodel_Data_Type[index].id).toString() : ''
                sensor_key += this.vmodel_Sensors[index] ? (this.vmodel_Sensors[index].id).toString() : ''
                type_alert += this.vmodel_Types[index] ? (this.vmodel_Types[index].id).toString() : ''

                limit_inf_str += limit_inf[index] == null ? '' : (limit_inf[index]).toString()
                limit_sup_str += limit_sup[index] == null ? '' : (limit_sup[index]).toString()

                if( index != this.arr_conditions.length - 1 ){
                    iddata_type += ';'
                    sensor_key += ';'
                    type_alert += ';'
                    limit_inf_str += ';'
                    limit_sup_str += ';'
                } else {
                    iddata_type = this.vmodel_Data_Type[index] ? iddata_type : null
                    sensor_key = this.vmodel_Sensors[index] ? sensor_key : null
                }

            })

            let formula = ''

            this.arr_conditions.forEach((condition, index) => {

                let key;

                if(iddata_type != null){
                    key = this.vmodel_Data_Type[index].id
                }else if(sensor_key != null){
                    key = this.vmodel_Sensors[index].id
                }

                if(limit_inf[index] != null && limit_sup[index] != null){

                    if(this.vmodel_Types[index].name == 'Threshold'){

                        let limit_sup_var = limit_sup[index]
                        let limit_inf_var = limit_inf[index]
                        formula += '((#new_value_' + key + '#) &lt; (' + limit_sup_var + ')) && ((#new_value_' + key + '#) &gt; (' + limit_inf_var + '))'

                    }else if(this.vmodel_Types[index].name == 'Variation'){

                        let limit_sup_var = 1 + parseFloat(limit_sup[index]/100)
                        let limit_inf_var = 1 - parseFloat(limit_inf[index]/100)
                        formula += '((#new_value_' + key + '#) &lt; (#old_value_' + key + '# * (' + limit_sup_var + '))) && ((#new_value_' + key + '#) &gt; (#old_value_' + key + '# * (' + limit_inf_var + ')))'

                    }

                }else if(limit_sup[index] != null){

                    if(this.vmodel_Types[index].name == 'Threshold'){

                        let limit_sup_var = limit_sup[index]
                        formula += '((#new_value_' + key + '#) &lt; (' + limit_sup_var + '))'

                    }else if(this.vmodel_Types[index].name == 'Variation'){

                        let limit_sup_var = 1 + parseFloat(limit_sup[index]/100)
                        formula += '((#new_value_' + key + '#) &lt; (#old_value_' + key + '# * (' + limit_sup_var + ')))'

                    }

                }else if(limit_inf[index] != null){

                    if(this.vmodel_Types[index].name == 'Threshold'){

                        let limit_inf_var = limit_inf[index]
                        formula += '((#new_value_' + key + '#) &gt; (' + limit_inf_var + '))'

                    }else if(this.vmodel_Types[index].name == 'Variation'){

                        let limit_inf_var = 1 - parseFloat(limit_inf[index]/100)
                        formula += '((#new_value_' + key + '#) &gt; (#old_value_' + key + '# * (' + limit_inf_var + ')))'

                    }else if(this.vmodel_Types[index].name == 'State'){

                        let limit_inf_var = limit_inf[index]
                        formula += '((#new_value_' + key + '#) == (' + limit_inf_var + '))'

                    }else if(this.vmodel_Types[index].name == 'Difference'){

                        let limit_inf_var = limit_inf[index]
                        formula += '((#new_value_' + key + '#) != (' + limit_inf_var + '))'

                    }

                }

                if( index != this.arr_conditions.length - 1 ){
                    /*if(!document.getElementById('id_conditional_selector').checked){
                        formula += ' || ';
                    }else if(document.getElementById('id_conditional_selector').checked){*/
                        formula += ' && ';
                    //}
                }

            })

            const data = {
                iddevice: this.vmodel_Devices.id,
                description: document.getElementById('id_alert_description').value,
                email: document.getElementById('id_alert_email').value,
                limit_inf: limit_inf_str,
                limit_sup: limit_sup_str,
                type_alert: type_alert,
                iddata_type: iddata_type,
                formula: formula,
            };

            this.gateways.forEach((gateway, index) => {
                if(this.vmodel_Devices){
                    if (gateway.id == this.vmodel_Devices.id) {
                        if (gateway.type_name.indexOf('MODBUS') != -1){
                            data.sensor_key = sensor_key;
                        } else if (gateway.type_name.indexOf('PULSES') != -1) {
                            data.BUS_iddevice = sensor_key;
                        }
                    }
                }
            })

            window.emitter.emit('changeLoaderStatus', true, 'orbit')

            axios({
                method: 'put',
                url: window.location.origin + '/api/alert/edit/' + this.user.id + '/' + this.alerts[this.temporal_alert_id].id,
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + this.user.token
                },
                data: data
            })
                .then((response) => {

                })
                .catch((error) => {
                    window.emitter.emit('changeLoaderStatus', false, 'orbit')

                    if(error.response && error.response.status && error.response.status == 429){
                        window.alert( this.$t('text.Api attempts'));
                    }else{
                        return "Failed!", error;
                    }
                });

            this.alerts[this.temporal_alert_id].iddevice = data.iddevice
            this.alerts[this.temporal_alert_id].description = data.description
            this.alerts[this.temporal_alert_id].sensor_key = this.Str_to_Array((typeof data.sensor_key != 'undefined') ? data.sensor_key : data.BUS_iddevice)
            this.alerts[this.temporal_alert_id].email = data.email
            this.alerts[this.temporal_alert_id].limit_inf = this.Str_to_Array(data.limit_inf)
            this.alerts[this.temporal_alert_id].limit_sup = this.Str_to_Array(data.limit_sup)
            this.alerts[this.temporal_alert_id].iddata_type = this.Str_to_Array(data.iddata_type)
            this.alerts[this.temporal_alert_id].type_alert = this.Str_to_Array(data.type_alert)

            window.emitter.emit('changeLoaderStatus', false, 'orbit')

            this.Edit_Modal = false;
            this.temporal_alert_id = null;

        },

        cancel_EditAlert() {
            this.Edit_Modal = false;
            this.temporal_alert_id = null;
            this.temporal_alert_limit_inf = null;
            this.temporal_alert_limit_sup = null;
        },

        // opens modal to create alert
        open_create_Modal() {
            this.Create_Modal = true;

            this.options_Devices = [];
            this.gateways.forEach((value, index, array) => {

                let obj = {
                    name: value.name, //+ ' (' + value.id + ')',
                    id: value.id
                }

                this.options_Devices.push(obj)

            })

            this.vmodel_Devices = null

            this.options_Types = [];
            this.alerts_type.forEach((type, key) => {

                let obj = {
                    name: type.description,
                    id: type.id,
                }

                this.options_Types.push(obj)
            })

            this.arr_conditions = new Array(1).fill(true);

            this.vmodel_Sensors = new Array(this.arr_conditions.length)
            this.vmodel_Data_Type = new Array(this.arr_conditions.length)
            this.vmodel_Types = new Array(this.arr_conditions.length)

            setTimeout(() => {
                this.enableButton()
            }, 0);
        },

        save_CreateAlert() {

            let limit_inf = new Array();
            let limit_sup = new Array();

            let sensor_key = '';
            let type_alert = '';
            let iddata_type = '';

            let limit_inf_str = '';
            let limit_sup_str = '';

            this.arr_conditions.forEach((condition, index) => {

                limit_inf.push(document.getElementById('id_alert_limit_inf_'+index) ? (document.getElementById('id_alert_limit_inf_'+index).value == '' ? null : document.getElementById('id_alert_limit_inf_'+index).value) : null)
                limit_sup.push(document.getElementById('id_alert_limit_sup_'+index) ? (document.getElementById('id_alert_limit_sup_'+index).value == '' ? null : document.getElementById('id_alert_limit_sup_'+index).value) : null)

                iddata_type += this.vmodel_Data_Type[index] ? (this.vmodel_Data_Type[index].id).toString() : ''
                sensor_key += this.vmodel_Sensors[index] ? (this.vmodel_Sensors[index].id).toString() : ''
                type_alert += this.vmodel_Types[index] ? (this.vmodel_Types[index].id).toString() : ''

                limit_inf_str += limit_inf[index] == null ? '' : (limit_inf[index]).toString()
                limit_sup_str += limit_sup[index] == null ? '' : (limit_sup[index]).toString()

                if( index != this.arr_conditions.length - 1 ){
                    iddata_type += ';'
                    sensor_key += ';'
                    type_alert += ';'
                    limit_inf_str += ';'
                    limit_sup_str += ';'
                } else {
                    iddata_type = this.vmodel_Data_Type[index] ? iddata_type : null
                    sensor_key = this.vmodel_Sensors[index] ? sensor_key : null
                }

            })

            let formula = ''

            this.arr_conditions.forEach((condition, index) => {

                let key;

                if(iddata_type != null){
                    key = this.vmodel_Data_Type[index].id
                }else if(sensor_key != null){
                    key = this.vmodel_Sensors[index].id
                }

                if(limit_inf[index] != null && limit_sup[index] != null){

                    if(this.vmodel_Types[index].name == 'Threshold'){

                        let limit_sup_var = limit_sup[index]
                        let limit_inf_var = limit_inf[index]
                        formula += '((#new_value_' + key + '#) &lt; (' + limit_sup_var + ')) && ((#new_value_' + key + '#) &gt; (' + limit_inf_var + '))'

                    }else if(this.vmodel_Types[index].name == 'Variation'){

                        let limit_sup_var = 1 + parseFloat(limit_sup[index]/100)
                        let limit_inf_var = 1 - parseFloat(limit_inf[index]/100)
                        formula += '((#new_value_' + key + '#) &lt; (#old_value_' + key + '# * (' + limit_sup_var + '))) && ((#new_value_' + key + '#) &gt; (#old_value_' + key + '# * (' + limit_inf_var + ')))'

                    }

                }else if(limit_sup[index] != null){

                    if(this.vmodel_Types[index].name == 'Threshold'){

                        let limit_sup_var = limit_sup[index]
                        formula += '((#new_value_' + key + '#) &lt; (' + limit_sup_var + '))'

                    }else if(this.vmodel_Types[index].name == 'Variation'){

                        let limit_sup_var = 1 + parseFloat(limit_sup[index]/100)
                        formula += '((#new_value_' + key + '#) &lt; (#old_value_' + key + '# * (' + limit_sup_var + ')))'

                    }

                }else if(limit_inf[index] != null){

                    if(this.vmodel_Types[index].name == 'Threshold'){

                        let limit_inf_var = limit_inf[index]
                        formula += '((#new_value_' + key + '#) &gt; (' + limit_inf_var + '))'

                    }else if(this.vmodel_Types[index].name == 'Variation'){

                        let limit_inf_var = 1 - parseFloat(limit_inf[index]/100)
                        formula += '((#new_value_' + key + '#) &gt; (#old_value_' + key + '# * (' + limit_inf_var + ')))'

                    }else if(this.vmodel_Types[index].name == 'State'){

                        let limit_inf_var = limit_inf[index]
                        formula += '((#new_value_' + key + '#) == (' + limit_inf_var + '))'

                    }else if(this.vmodel_Types[index].name == 'Difference'){

                        let limit_inf_var = limit_inf[index]
                        formula += '((#new_value_' + key + '#) != (' + limit_inf_var + '))'

                    }

                }

                if( index != this.arr_conditions.length - 1 ){
                    /*if(!document.getElementById('id_conditional_selector').checked){
                        formula += ' || ';
                    }else if(document.getElementById('id_conditional_selector').checked){*/
                        formula += ' && ';
                    //}
                }

            })

            const data = {
                iddevice: this.vmodel_Devices.id,
                description: document.getElementById('id_alert_description').value,
                email: document.getElementById('id_alert_email').value,
                limit_inf: limit_inf_str,
                limit_sup: limit_sup_str,
                type_alert: type_alert,
                iddata_type: iddata_type,
                formula: formula,
                start_at: this.getUTC_Time({hour: 0, min: 0}, 0),
                end_at: this.getUTC_Time({hour: 23, min: 59}, 59),
            };

            this.gateways.forEach((gateway, index) => {
                if(this.vmodel_Devices){
                    if (gateway.id == this.vmodel_Devices.id) {
                        if (gateway.type_name.indexOf('MODBUS') != -1 || gateway.MODBUS_device_type != ''){
                            data.sensor_key = sensor_key;
                        } else if (gateway.type_name.indexOf('PULSES') != -1) {
                            data.BUS_iddevice = sensor_key;
                        }
                    }
                }
            })

            window.emitter.emit('changeLoaderStatus', true, 'orbit')

            axios({
                method: 'post',
                url: window.location.origin + '/api/alert/create/' + this.user.id + '/' + this.user.idgroup_selected,
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + this.user.token
                },
                data: data
            })
                .then((response) => {

                    this.alerts.push({
                        id: response.data[0].id,
                        iddevice: data.iddevice,
                        description: data.description,
                        sensor_key: this.Str_to_Array((typeof data.sensor_key != 'undefined') ? data.sensor_key : data.BUS_iddevice),
                        email: data.email,
                        limit_inf: this.Str_to_Array(data.limit_inf),
                        limit_sup: this.Str_to_Array(data.limit_sup),
                        type_alert: this.Str_to_Array(data.type_alert),
                        iddata_type: this.Str_to_Array(data.iddata_type),
                        enabled: 1,
                        start_at: '00:00',
                        end_at: '23:59',
                        enabledDays: JSON.stringify({"0":1,"1":1,"2":1,"3":1,"4":1,"5":1,"6":1}),
                    })

                    window.emitter.emit('changeLoaderStatus', false, 'orbit')

                })
                .catch((error) => {
                    window.emitter.emit('changeLoaderStatus', false, 'orbit')

                    if(error.response && error.response.status && error.response.status == 429){
                        window.alert( this.$t('text.Api attempts'));
                    }else{
                        return "Failed!", error;
                    }
                });

            this.Create_Modal = false;
            this.enableButtonvar = false;

        },

        cancel_CreateAlert() {
            this.Create_Modal = false;
            this.enableButtonvar = false;
        },

        // sets the badge number
        setAlertsTextNavbar() {

            localStorage.setItem("Alerts", 0);

            for (var index = 0; index < this.alerts_log.length; index++) {
                if (this.alerts_log[index].read == 0) {
                    localStorage.setItem("Alerts", +localStorage.getItem("Alerts") + 1);
                }
            }

            if (+localStorage.getItem("Alerts") > 9) {
                localStorage.setItem("Alerts", '9+');
            }

        },

        // Sets temporal vars to selected alert in order to filter with his description
        set_Alert_filter(index) {

            var alerts_log = this.alerts_log
            var temporal_display_filter_alerts = this.temporal_display_filter_alerts
            var alerts = this.alerts

            if (this.temporal_alert_id != index) {

                this.temporal_alert_id = index;
                temporal_display_filter_alerts.fill(true)

                for (var index = 0; index < alerts_log.length; index++) {

                    if (alerts[this.temporal_alert_id].description != alerts_log[index].description) {

                        temporal_display_filter_alerts[index] = false

                    }

                }

                this.update_Filters();

            } else if (!this.Edit_Modal) {

                this.temporal_alert_id = null;
                temporal_display_filter_alerts.fill(true)
                this.update_Filters();

            }

        },

        // Opens delete modal confirmation and deletes the 'selector' item when 'final_delete' is true
        open_Delete(final_delete, selector, index) {

            if (final_delete && selector == 'alert') {

                this.delete_Alert()

                this.Delete_Modal = false;

            } else if (final_delete && selector == 'alert_log') {

                this.delete_AlertLog(this.Delete_index)

                this.Delete_Modal = false;

            } else if (final_delete && selector == 'group_alert_log') {

                this.delete_SelectedAlertLogs()

                this.Delete_Modal = false;

            } else if (final_delete && selector == 'condition') {

                this.PopCondition(this.Delete_index)

                this.Delete_Modal = false;

            } else {

                this.Delete_Modal = true;

                if (typeof index != 'undefined' && (selector == 'alert_log' || selector == 'condition')) {
                    this.Delete_index = index
                }
            }

            this.temporal_selector_delete = selector;
        },

        // Closes the delete confirmation modal
        close_Delete() {
            this.Delete_Modal = false;
        },

        // Deletes the selected alert
        delete_Alert() {

            this.temporal_alert_deleted.push(this.temporal_alert_id)

            window.emitter.emit('changeLoaderStatus', true, 'orbit')

            axios({
                method: 'delete',
                url: window.location.origin + '/api/alert/delete/' + this.user.id + '/' + this.alerts[this.temporal_alert_id].id,
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + this.user.token
                },
            })
                .then((response) => {
                    window.emitter.emit('changeLoaderStatus', false, 'orbit')
                    return ("Success!", response);
                })
                .catch((error) => {
                    window.emitter.emit('changeLoaderStatus', false, 'orbit')

                    if(error.response && error.response.status && error.response.status == 429){
                        window.alert( this.$t('text.Api attempts'));
                    }else{
                        return "Failed!", error;
                    }
                });

            this.temporal_display_filter_alerts.fill(true)
            this.update_Filters()

            this.Edit_Modal = false;
            this.temporal_alert_id = null;
            this.Create_Modal = false;

        },

        //
        switch_AlertEnable(id_alert) {

            let enabled = document.getElementById('id_switch_' + id_alert).checked
            let alerts = this.alerts
            let user = this.user

            alerts.forEach(function callback(alert, index) {
                if (alert.id == id_alert && (alert.enabled == 1) != enabled) {
                    enabled ? alerts[index].enabled = 1 : alerts[index].enabled = 0

                    axios({
                        method: 'put',
                        url: window.location.origin + '/api/alert/edit_enabled/' + user.id + '/' + id_alert + '/' + enabled,
                        headers: {
                            'Accept': 'application/json',
                            'Content-Type': 'application/json',
                            'Authorization': 'Bearer ' + user.token
                        },
                    })
                        .then((response) => {
                            return ("Success!", response);
                        })
                        .catch((error) => {
                            window.emitter.emit('changeLoaderStatus', false, 'orbit')

                            if(error.response && error.response.status && error.response.status == 429){
                                window.alert( this.$t('text.Api attempts'));
                            }else{
                                return "Failed!", error;
                            }
                        });

                }
            });
        },

        // FOR THE LOGS

        // Sets the checkboxes and temporal_selected_logs global var when we click a checkbox
        set_CheckboxClick(index, selector) {

            if (index == 'checkbox_all') {

                if (document.getElementById('checkbox_all').checked == true) {

                    var temporal_selected_logs = this.temporal_selected_logs;

                    this.alerts_log.forEach(function callback(log, index) {
                        temporal_selected_logs[index] = document.getElementById('checkbox_' + log.id).checked;
                    });

                }

                if (selector == 'All_box') {

                    if (document.getElementById('checkbox_all').checked) {

                        document.getElementById('checkbox_all').checked = true;

                        this.alerts_log.forEach(log => {
                            document.getElementById('checkbox_' + log.id).checked = true;
                        });

                    } else {

                        document.getElementById('checkbox_all').checked = false;

                        this.alerts_log.forEach(log => {
                            document.getElementById('checkbox_' + log.id).checked = false;
                        });

                    }

                } else if (selector == 'All') {

                    document.getElementById('checkbox_all').checked = true;

                    this.alerts_log.forEach(log => {
                        document.getElementById('checkbox_' + log.id).checked = true;
                    });

                } else if (selector == 'None') {

                    document.getElementById('checkbox_all').checked = false;

                    this.alerts_log.forEach(log => {
                        document.getElementById('checkbox_' + log.id).checked = false;
                    });

                } else if (selector == 'Read') {

                    document.getElementById('checkbox_all').checked = true;

                    this.alerts_log.forEach(log => {

                        if (log.read == 1)
                            document.getElementById('checkbox_' + log.id).checked = true;
                        else
                            document.getElementById('checkbox_' + log.id).checked = false;

                    });

                } else if (selector == 'Unread') {

                    document.getElementById('checkbox_all').checked = true;

                    this.alerts_log.forEach(log => {

                        if (log.read == 0)
                            document.getElementById('checkbox_' + log.id).checked = true;
                        else
                            document.getElementById('checkbox_' + log.id).checked = false;

                    });

                }

            } else {

                document.getElementById('checkbox_all').checked = false;

            }

            this.Global_Checkbox_State = document.getElementById('checkbox_all').checked;

            this.Checkboxes_State = false;

            this.alerts_log.forEach(log => {

                if (document.getElementById('checkbox_' + log.id).checked)
                    this.Checkboxes_State = true;

            });

            this.setAlertsTextNavbar()
        },

        // Resets temporal_selected_logs to the previous selection
        back_Checkboxes() {

            this.alerts_log.forEach(function callback(log, index) {
                document.getElementById('checkbox_' + log.id).checked = false;
            });

            document.getElementById('checkbox_all').checked = false;
            this.Global_Checkbox_State = document.getElementById('checkbox_all').checked;

            this.Checkboxes_State = false;

            this.alerts_log.forEach(log => {

                if (document.getElementById('checkbox_' + log.id).checked)
                    this.Checkboxes_State = true;

            });

        },

        // Marks as read the selected log
        set_ReadAlertLog(index) {

            if (this.alerts_log[index].read != 1) {
                this.alerts_log[index].read = 1;

                //axios.post('/api/alert_log/edit/' + this.user.token + '/' + this.alerts_log[index].id + '/' + this.alerts_log[index].read);

                axios({
                    method: 'put',
                    url: window.location.origin + '/api/alert_log/edit/' + this.user.id + '/' + this.alerts_log[index].id + '/' + this.alerts_log[index].read,
                    headers: {
                        'Accept': 'application/json',
                        'Content-Type': 'application/json',
                        'Authorization': 'Bearer ' + this.user.token
                    },
                })
                    .then((response) => {
                        return ("Success!", response);
                    })
                    .catch((error) => {
                        window.emitter.emit('changeLoaderStatus', false, 'orbit')

                        if(error.response && error.response.status && error.response.status == 429){
                            window.alert( this.$t('text.Api attempts'));
                        }else{
                            return "Failed!", error;
                        }
                    });

                this.setAlertsTextNavbar()

            }
        },

        // Deletes the index log
        delete_AlertLog(index) {

            this.temporal_display_deleted[index] = false

            window.emitter.emit('changeLoaderStatus', true, 'orbit')

            axios({
                method: 'delete',
                url: window.location.origin + '/api/alert_log/delete/' + this.user.id + '/' + this.alerts_log[index].id,
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + this.user.token
                },
            })
                .then((response) => {
                    window.emitter.emit('changeLoaderStatus', false, 'orbit')
                    return ("Success!", response);
                })
                .catch((error) => {
                    window.emitter.emit('changeLoaderStatus', false, 'orbit')

                    if(error.response && error.response.status && error.response.status == 429){
                        window.alert( this.$t('text.Api attempts'));
                    }else{
                        return "Failed!", error;
                    }
                });

            this.update_Filters()

        },

        // Marks as read the selected logs
        set_ReadSelectedAlertLogs(select) {

            var user = this.user

            this.alerts_log.forEach((log) => {

                if (document.getElementById('checkbox_' + log.id).checked) {
                    //log.read = select ? 1 : 0;

                    if (select) {
                        if (log.read != 1) {
                            log.read = 1
                            //axios.post('/api/alert_log/edit/' + user.token + '/' + log.id + '/' + log.read);

                            axios({
                                method: 'put',
                                url: window.location.origin + '/api/alert_log/edit/' + user.id + '/' + log.id + '/' + log.read,
                                headers: {
                                    'Accept': 'application/json',
                                    'Content-Type': 'application/json',
                                    'Authorization': 'Bearer ' + this.user.token
                                },
                            })
                                .then((response) => {
                                    return ("Success!", response);
                                })
                                .catch((error) => {
                                    window.emitter.emit('changeLoaderStatus', false, 'orbit')

                                    if(error.response && error.response.status && error.response.status == 429){
                                        window.alert( this.$t('text.Api attempts'));
                                    }else{
                                        return "Failed!", error;
                                    }
                                });
                        }
                    } else {
                        if (log.read != 0) {
                            log.read = 0
                            //axios.post('/api/alert_log/edit/' + user.token + '/' + log.id + '/' + log.read);

                            axios({
                                method: 'put',
                                url: window.location.origin + '/api/alert_log/edit/' + user.id + '/' + log.id + '/' + log.read,
                                headers: {
                                    'Accept': 'application/json',
                                    'Content-Type': 'application/json',
                                    'Authorization': 'Bearer ' + this.user.token
                                },
                            })
                                .then((response) => {
                                    return ("Success!", response);
                                })
                                .catch((error) => {
                                    window.emitter.emit('changeLoaderStatus', false, 'orbit')

                                    if(error.response && error.response.status && error.response.status == 429){
                                        window.alert( this.$t('text.Api attempts'));
                                    }else{
                                        return "Failed!", error;
                                    }
                                });

                        }
                    }
                }
            });

            this.setAlertsTextNavbar()

        },

        // Deletes the selected logs
        delete_SelectedAlertLogs() {

            var user = this.user
            var temporal_display_deleted = this.temporal_display_deleted

            this.alerts_log.forEach(function callback(log, index) {

                if (document.getElementById('checkbox_' + log.id).checked) {

                    temporal_display_deleted[index] = false

                    window.emitter.emit('changeLoaderStatus', true, 'orbit')

                    axios({
                        method: 'delete',
                        url: window.location.origin + '/api/alert_log/delete/' + user.id + '/' + log.id,
                        headers: {
                            'Accept': 'application/json',
                            'Content-Type': 'application/json',
                            'Authorization': 'Bearer ' + user.token
                        },
                    })
                        .then((response) => {
                            window.emitter.emit('changeLoaderStatus', false, 'orbit')
                            return ("Success!", response);
                        })
                        .catch((error) => {
                            window.emitter.emit('changeLoaderStatus', false, 'orbit')

                            if(error.response && error.response.status && error.response.status == 429){
                                window.alert( this.$t('text.Api attempts'));
                            }else{
                                return "Failed!", error;
                            }
                        });

                }
            });

            this.update_Filters()

        },

        // OTHERS (format, filters, ...)

        // Updates logs depending on the filters
        update_Filters() {

            var temporal_display_logs = this.temporal_display_logs

            for (var index = 0; index < temporal_display_logs.length; index++) {
                temporal_display_logs[index] =
                    this.temporal_display_deleted[index] &&
                    this.temporal_display_filter_search[index] &&
                    this.temporal_display_filter_dates[index] &&
                    this.temporal_display_filter_alerts[index]
            }

        },

        // Formating the dp dates
        Format() {
            var date = this.date;
            var to = this.$t("general." + "To")

            var day1 = date[0].getDate();
            var month1 = date[0].getMonth() + 1;
            var year1 = date[0].getFullYear();
            var hour1 = date[0].getHours();
            var min1 = date[0].getMinutes();

            if (month1.toString().length == 1) month1 = '0' + month1;
            if (day1.toString().length == 1) day1 = '0' + day1;
            if (hour1.toString().length == 1) hour1 = '0' + hour1;
            if (min1.toString().length == 1) min1 = '0' + min1;

            var day2 = date[1].getDate();
            var month2 = date[1].getMonth() + 1;
            var year2 = date[1].getFullYear();
            var hour2 = date[1].getHours();
            var min2 = date[1].getMinutes();

            if (month2.toString().length == 1) month2 = '0' + month2;
            if (day2.toString().length == 1) day2 = '0' + day2;
            if (hour2.toString().length == 1) hour2 = '0' + hour2;
            if (min2.toString().length == 1) min2 = '0' + min2;

            if(this.screen_width >= 768){
                return this.DP_open ? `${year1}/${month1}/${day1} ${hour1}:${min1} ${to} ${year2}/${month2}/${day2} ${hour2}:${min2}` : '';
            }else{
                return this.DP_open ? `${month1}/${day1} ${hour1}:${min1} ${to} ${month2}/${day2} ${hour2}:${min2}` : '';
            }


        },

        //Gets first and last date
        get_InitialDates(selector) {
            if (this.alerts_log[0]) {
                if (selector == 'first') {
                    return new Date(this.alerts_log[this.alerts_log.length - 1].date);
                } else if (selector == 'last') {
                    return new Date(this.alerts_log[0].date);
                }
            } else {
                return new Date();
            }
        },

        //Deletes dates filter
        delete_FilterDates() {

            this.DP_open = false;
            this.date = this.initial_date;
            this.temporal_display_filter_dates.fill(true)

            this.update_Filters();

        },

        // method that is called when we update dates and sets the dates filters
        set_FilterDates(modelData) {

            if (!this.DP_open) {
                for (var i = 0; i < 2; i++) {
                    this.initial_date[i] = this.date[i];
                }
            }

            this.DP_open = true;

            var alerts_log = this.alerts_log;

            var start_date = new Date((modelData[0]));
            start_date = start_date.getTime() - this.user.timezone_offset * 1000 * 60 * 60;

            var end_date = new Date((modelData[1]));
            end_date = end_date.getTime() - this.user.timezone_offset * 1000 * 60 * 60;

            var end_event = new Date(end_date);

            var start_event = new Date(start_date);

            //////////////////////

            var temporal_display_filter_dates = this.temporal_display_filter_dates

            temporal_display_filter_dates.fill(true)

            for (var index = 0; index < alerts_log.length; index++) {

                // dates filter
                var log_event = new Date(alerts_log[index].date);

                if (start_event > log_event || log_event > end_event) {

                    temporal_display_filter_dates[index] = false

                }

            }

            this.update_Filters();

            //////////////////////

            end_event = new Date(end_event.getTime() + this.user.timezone_offset * 1000 * 60 * 60); // + new Date().getTimezoneOffset() * 60 * 1000 ?

            end_event.setDate(end_event.getDate());
            end_event.setMonth(end_event.getMonth());
            end_event.setHours(end_event.getHours());

            var MM = end_event.getMonth() + 1;
            var dd = end_event.getDate();
            var hh = end_event.getHours();
            var mm = end_event.getMinutes();
            var ss = end_event.getSeconds();

            if (MM.toString().length == 1) MM = '0' + MM;
            if (dd.toString().length == 1) dd = '0' + dd;
            if (hh.toString().length == 1) hh = '0' + hh;
            if (mm.toString().length == 1) mm = '0' + mm;
            if (ss.toString().length == 1) ss = '0' + ss;

            end_date = end_event.getFullYear() + '-' + MM + '-' + dd + ' ' + hh + ':' + mm + ':' + ss;


            start_event = new Date(start_event.getTime() + this.user.timezone_offset * 1000 * 60 * 60);

            start_event.setDate(start_event.getDate());
            start_event.setMonth(start_event.getMonth());
            start_event.setHours(start_event.getHours());

            var MM = start_event.getMonth() + 1;
            var dd = start_event.getDate();
            var hh = start_event.getHours();
            var mm = start_event.getMinutes();
            var ss = start_event.getSeconds();

            if (MM.toString().length == 1) MM = '0' + MM;
            if (dd.toString().length == 1) dd = '0' + dd;
            if (hh.toString().length == 1) hh = '0' + hh;
            if (mm.toString().length == 1) mm = '0' + mm;
            if (ss.toString().length == 1) ss = '0' + ss;

            start_date = start_event.getFullYear() + '-' + MM + '-' + dd + ' ' + hh + ':' + mm + ':' + ss;

            const startDate = new Date(start_date + ' GMT');
            const endDate = new Date(end_date + ' GMT');

            this.date = [startDate, endDate];

        },

        // Filtering by the Searchbar text
        set_SearchFilter() {
            var alerts_log = this.alerts_log
            var alerts_type = this.alerts_type
            var temporal_display_filter_search = this.temporal_display_filter_search
            var txtSearchbar, subfilters, txtValueDescription, txtValueType

            temporal_display_filter_search.fill(false)

            //text to filter
            txtSearchbar = document.getElementById("id_searchbar").value.toUpperCase();

            subfilters = txtSearchbar.split(",");

            if (txtSearchbar.trim() != '') {

                subfilters.forEach(filter => {

                    filter = filter.trim();

                    for (var index = 0; index < alerts_log.length; index++) {

                        // description search
                        txtValueDescription = alerts_log[index].description;

                        // type search
                        for (var index_type = 0; index_type < alerts_type.length; index_type++) {

                            if (alerts_type[index_type].id == alerts_log[index].type_alert) {
                                txtValueType = alerts_type[index_type].description
                            }

                        }

                        if (filter != '') {

                            if (txtValueDescription.toUpperCase().indexOf(filter) > -1 || txtValueType.toUpperCase().indexOf(filter) > -1) {
                                temporal_display_filter_search[index] = true
                            }

                        }

                    }

                });

            } else {

                temporal_display_filter_search.fill(true)

            }

            this.update_Filters();

        },

        AddCondition(){

            this.arr_conditions.push(true)
            this.vmodel_Sensors.push(undefined)
            this.vmodel_Data_Type.push(undefined)
            this.vmodel_Types.push(undefined)
            if(this.temporal_alert_limit_inf){this.temporal_alert_limit_inf.push(undefined)}
            if(this.temporal_alert_limit_sup){this.temporal_alert_limit_sup.push(undefined)}

            setTimeout(() => {
                this.enableButton()
            }, 0);
        },

        PopCondition(index){

            this.arr_conditions.splice(index, 1)
            this.vmodel_Sensors.splice(index, 1)
            this.vmodel_Data_Type.splice(index, 1)
            this.vmodel_Types.splice(index, 1)

            if(this.temporal_alert_limit_sup){this.temporal_alert_limit_sup.splice(index, 1)}
            if(this.temporal_alert_limit_inf){this.temporal_alert_limit_inf.splice(index, 1)}

            setTimeout(() => {
                this.enableButton()
            }, 0);
        },

        ShowCondition(index){
            this.arr_conditions[index] = !this.arr_conditions[index]
        },

        Str_to_Array(str) {
            // Dividir el string usando ";" como separador
            const valores = (str != null) ? str.split(';') : new Array(null)
            // Convertir los valores en el array
            const resultado = valores.map(valor => {
                // Convertir a número si no es "null", de lo contrario, mantener como null
                return valor !== '' ? parseFloat(valor) : null;
            });
            return resultado;
        },

        open_Timetable(index){

            this.Timetable_Modal = true;
            this.temporal_alert_id = index;

            var objetoDesdeJson = JSON.parse(this.alerts[index].enabledDays);

            var activeAlarmDays = Array.from({ length: 7 }, function(_, i) {
                return objetoDesdeJson[i.toString()] === 1;
            });

            this.activeAlarmDays = activeAlarmDays;

            var [hours, minutes] = this.alerts[index].start_at.split(":");

            this.start_at = {
                hour: parseInt(hours, 10),
                min: parseInt(minutes, 10)
            };

            var [hours, minutes] = this.alerts[index].end_at.split(":");

            this.end_at = {
                hour: parseInt(hours, 10),
                min: parseInt(minutes, 10)
            };

            this.EnableSaveHours = false;
        },

        getUTC_Time(Date_hour, seconds, format){

            // Crea un objeto Date con las horas y minutos locales
            let localTime = new Date();
            localTime.setHours(Date_hour.hour);
            localTime.setMinutes(Date_hour.min);

            // Ajusta la hora local según el desfase
            const utcTime = new Date(localTime.getTime() - (this.user.timezone_offset * 60 * 60 * 1000));

            // Obtén las horas, minutos y segundos en formato de cadena
            const hours = utcTime.getHours().toString().padStart(2, '0');
            const minutes = utcTime.getMinutes().toString().padStart(2, '0');

            if(format == undefined){ // Formatea la hora en UTC como una cadena con el formato "12:00:00"
                return `${hours}:${minutes}:${seconds.toString().padStart(2, '0')}`;
            }else if(format == 0){ // Formatea la hora en UTC como una cadena con el formato "12:00"
                return `${hours}:${minutes}`;
            }
        },

        save_Timetable(){

            let start_at = this.getUTC_Time(this.start_at, 0);
            let end_at = this.getUTC_Time(this.end_at, 59);
            let activeAlarmDays = {};

            this.activeAlarmDays.forEach( (day, index) => {
                activeAlarmDays[index] = day ? 1 : 0;
            })

            activeAlarmDays = JSON.stringify(activeAlarmDays)

            const data = {
                start_at: start_at,
                end_at: end_at,
                enabledDays: activeAlarmDays,
            };

            axios({
                method: 'put',
                url: window.location.origin + '/api/alert/edit_timetable/' + this.user.id + '/' + this.alerts[this.temporal_alert_id].id,
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + this.user.token
                },
                data: data
            })
            .then((response) => {
                // ALMACENAR LOS CAMBIOS CUANDO SE ACTUALIZA EN THIS.ALERTS

                this.alerts[this.temporal_alert_id].end_at = (this.end_at.hour).toString().padStart(2, '0') + ':' + (this.end_at.min).toString().padStart(2, '0')
                this.alerts[this.temporal_alert_id].start_at = (this.start_at.hour).toString().padStart(2, '0') + ':' + (this.start_at.min).toString().padStart(2, '0')
                this.alerts[this.temporal_alert_id].enabledDays = activeAlarmDays

            })
            .catch((error) => {
                window.emitter.emit('changeLoaderStatus', false, 'orbit')

                if(error.response && error.response.status && error.response.status == 429){
                    window.alert( this.$t('text.Api attempts'));
                }else{
                    return "Failed!", error;
                }
            });

            this.Timetable_Modal = false;
            this.enableButtonvar = false;
        },

        cancel_Timetable(){
            this.Timetable_Modal = false;
            this.enableButtonvar = false;
        },

        GetDay(index){
            switch(index){
                case 0: return 'Sun';
                case 1: return 'Mon';
                case 2: return 'Tue';
                case 3: return 'Wed';
                case 4: return 'Thu';
                case 5: return 'Fri';
                case 6: return 'Sat';
            }
        },

        SwitchDayActive(index, case_num){

            if(typeof index == 'number'){
                this.activeAlarmDays[index] = !this.activeAlarmDays[index];
            }else{

                let caseIndexes = this.GetCaseIndexes(case_num);
                let caseState = this.CheckDayActive(caseIndexes)

                if(caseState.value == true){
                    caseIndexes.forEach((sub_index) => {
                        this.activeAlarmDays[sub_index] = !caseState.equals;
                    })
                }else{
                    caseIndexes.forEach((sub_index) => {
                        this.activeAlarmDays[sub_index] = true;
                    })
                }
            }
        },

        CheckDayActive(caseIndexes){

            let temp_bool = this.activeAlarmDays[caseIndexes[0]];
            let equals_bool = true;

            caseIndexes.forEach((sub_index) => {
                if(this.activeAlarmDays[sub_index] != temp_bool){
                    equals_bool = false
                }
            })

            return {
                equals: equals_bool,
                value: temp_bool,
            }
        },

        GetCaseIndexes(case_num){

            switch(case_num){
                case 0:
                    return new Array(1, 2, 3, 4, 5);

                case 1:
                    return new Array(6, 0);

                default:
                    return [];
            }

        },

        validarHora() {
            if (this.start_at.hour < 0) {
                this.start_at.hour = 0;
            } else if (this.start_at.hour > 23) {
                this.start_at.hour = 23;
            }

            if (this.end_at.hour < 0) {
                this.end_at.hour = 0;
            } else if (this.end_at.hour > 23) {
                this.end_at.hour = 23;
            }

            //this.validarOrden()
        },

        validarMinuto() {
            if (this.start_at.min < 0) {
                this.start_at.min = 0;
            } else if (this.start_at.min > 59) {
                this.start_at.min = 59;
            }

            if (this.end_at.min < 0) {
                this.end_at.min = 0;
            } else if (this.end_at.min > 59) {
                this.end_at.min = 59;
            }

            //this.validarOrden()
        },

        validarOrden(){
            if(this.end_at.hour * 60 + this.end_at.min <= this.start_at.hour * 60 + this.start_at.min){
                this.end_at.hour = this.start_at.hour;
                this.end_at.min = this.start_at.min + 1;
            }
        },

        GetQuarterHourColor(index){
            //array con 24*4 posiciones
            let end_at = this.end_at.hour * 4 + Math.round(this.end_at.min/15);
            let start_at = this.start_at.hour * 4 + Math.round(this.start_at.min/15);

            if(end_at > start_at){
                if(start_at <= index && index <= end_at){
                    return true;
                }else{
                    return false;
                }
            }else if(end_at < start_at){
                if(start_at <= index || index <= end_at){
                    return true;
                }else{
                    return false;
                }
            }else{
                return true;
            }
        }
    },

}
</script>
